{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {
    "cell_id": "95ce3f1c142f4d4aa372ad1eef397d5a",
    "deepnote_cell_type": "markdown",
    "tags": []
   },
   "source": [
    "# Week 6 Wednesday"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "cell_id": "676d36e819ad45eb9c9500942c2ae652",
    "deepnote_cell_type": "markdown",
    "tags": []
   },
   "source": [
    "## Announcements\n",
    "\n",
    "* My usual office hours today are cancelled.  Replaced by today 11am next door in ALP 3610 (I think it will be available).\n",
    "* No curving on the midterm (Median 41.6, Mean 41.3) but if you score higher on Midterm 2 (Week 9), we will weight that midterm 35% and this midterm 15% (instead of 25% and 25%).\n",
    "* I forgot to post solutions to the midterm, will post them Friday!\n",
    "* Videos and video quizzes posted; due Friday before lecture."
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "cell_id": "229ddab31b494329b09f340aa3c5a971",
    "deepnote_cell_type": "markdown",
    "tags": []
   },
   "source": [
    "## The K-means algorithm\n",
    "\n",
    "Copied from Monday:\n",
    "\n",
    "We choose the number of clusters we want to search for, `n_clusters`.  For this example, let's say `n_clusters=3` and let's say there are 500 points in the dataset.\n",
    "\n",
    "1.  Randomly choose 3 points, called *centroids*.\n",
    "2.  For each of the 500 points in the dataset, assign it to the nearest centroid.  We have now divided the data into 3 clusters.\n",
    "3.  Compute the centroids (also called averages, also called means) of each of these 3 clusters.  These 3 centroids are what is meant by the phrase *K-means* (in this case, `K = 3`).\n",
    "4.  For each of the 500 points in the dataset, assign it to the nearest centroid.  Continue repeating this process (assign to a cluster, compute the centroid of each cluster) until the process terminates. (By terminates, we mean each point remains in the same cluster.)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {
    "cell_id": "b2641f11efdd4d1790e6eb9c751685e5",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 1,
    "execution_start": 1667408502470,
    "source_hash": "48934de4",
    "tags": []
   },
   "outputs": [],
   "source": [
    "import numpy as np\n",
    "import pandas as pd\n",
    "import altair as alt\n",
    "\n",
    "from sklearn.datasets import make_blobs"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "cell_id": "4f0330a07a5a4370a295f95d307a1864",
    "deepnote_cell_type": "markdown",
    "tags": []
   },
   "source": [
    "## Code from Monday\n",
    "\n",
    "Here is the final code we used on Monday.  Notice that we use `centers=4` in `make_blobs`, which means there are 4 true centers, but we use 3 points in our `centroids` array (that is our guess for the number of centers)."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {
    "cell_id": "7181db5c2d744ae9935b6c486f23a4d4",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 2,
    "execution_start": 1667408518772,
    "source_hash": "86056a0a",
    "tags": []
   },
   "outputs": [],
   "source": [
    "# Notice centers does not equal n_clusters\n",
    "X, _ = make_blobs(n_samples=500, n_features=2, centers=4, random_state=1)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {
    "cell_id": "01fba96032214437ab5efce7ec1bb429",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 1,
    "execution_start": 1667408520793,
    "source_hash": "2089848b",
    "tags": []
   },
   "outputs": [],
   "source": [
    "centroids = np.array([[-4, -10], [-4, -8], [-10, -10]])\n",
    "df_centroids = pd.DataFrame(centroids, columns=[\"x\",\"y\"])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {
    "cell_id": "01351294285c459c8db287d7385da7b9",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 1,
    "execution_start": 1667408521843,
    "source_hash": "af5df1c3",
    "tags": []
   },
   "outputs": [],
   "source": [
    "def closest_centroid_index(z, centroids):\n",
    "    return ((centroids - z)**2).sum(axis=1).argmin()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {
    "cell_id": "6052ab0f01a448ab8ce23d85953c7307",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 2,
    "execution_start": 1667408522956,
    "source_hash": "da448923",
    "tags": []
   },
   "outputs": [],
   "source": [
    "df_data = pd.DataFrame(X, columns=[\"x\", \"y\"])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {
    "cell_id": "bbe798ab39f74e7c98a58d42043857e0",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 14,
    "execution_start": 1667408523819,
    "source_hash": "5aabb0b5",
    "tags": []
   },
   "outputs": [],
   "source": [
    "df_data[\"cluster\"] = df_data.apply(lambda row: closest_centroid_index(row.values, centroids), axis=1)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {
    "cell_id": "7b5b9854c44b4a6a957eebd1234a1f86",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 26,
    "execution_start": 1667408526632,
    "source_hash": "55bd1a18",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<div id=\"altair-viz-944b5c6e213c4cbda68af6ef3c37e545\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-944b5c6e213c4cbda68af6ef3c37e545\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-944b5c6e213c4cbda68af6ef3c37e545\");\n",
       "    }\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm//vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm//vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm//vega-lite@4.17.0?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm//vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      require([\"vega-embed\"], displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"4.17.0\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 400, \"continuousHeight\": 300}}, \"layer\": [{\"data\": {\"name\": \"data-091c9f531e8e5fe5b1fd597af6f6cb1a\"}, \"mark\": \"circle\", \"encoding\": {\"color\": {\"field\": \"cluster\", \"type\": \"nominal\"}, \"x\": {\"field\": \"x\", \"type\": \"quantitative\"}, \"y\": {\"field\": \"y\", \"type\": \"quantitative\"}}}, {\"data\": {\"name\": \"data-d937417af026e4f584a32531c2ef6637\"}, \"mark\": {\"type\": \"point\", \"filled\": true, \"shape\": \"cross\", \"size\": 500, \"stroke\": \"black\"}, \"encoding\": {\"x\": {\"field\": \"x\", \"type\": \"quantitative\"}, \"y\": {\"field\": \"y\", \"type\": \"quantitative\"}}}], \"$schema\": \"https://vega.github.io/schema/vega-lite/v4.17.0.json\", \"datasets\": {\"data-091c9f531e8e5fe5b1fd597af6f6cb1a\": [{\"x\": -6.923241645991934, \"y\": -10.669531965589794, \"cluster\": 0}, {\"x\": -8.630620334313644, \"y\": -7.13940563765234, \"cluster\": 2}, {\"x\": -9.630480689769236, \"y\": -2.7204493549869673, \"cluster\": 2}, {\"x\": -2.3064765942034278, \"y\": 5.307976760491872, \"cluster\": 1}, {\"x\": -7.570053657727669, \"y\": -3.0144649054564128, \"cluster\": 1}, {\"x\": -1.0005101098483093, \"y\": 2.779051528226904, \"cluster\": 1}, {\"x\": -4.818268385973806, \"y\": -2.772148223435087, \"cluster\": 1}, {\"x\": -5.339647992855507, \"y\": -1.276257644021504, \"cluster\": 1}, {\"x\": -7.943088401134986, \"y\": -3.899939010564857, \"cluster\": 1}, {\"x\": -5.549245254367849, \"y\": -3.412989678624256, \"cluster\": 1}, {\"x\": -5.145089897229651, \"y\": -9.544921980678078, \"cluster\": 0}, {\"x\": -7.09669935635653, \"y\": -8.04074036220985, \"cluster\": 1}, {\"x\": -5.826415121664895, \"y\": -1.963461960474791, \"cluster\": 1}, {\"x\": -1.8319881134989553, \"y\": 3.5286314509217895, \"cluster\": 1}, {\"x\": -7.342672348933138, \"y\": -3.1654648218447767, \"cluster\": 1}, {\"x\": -7.340728246218882, \"y\": -6.924272518973348, \"cluster\": 1}, {\"x\": -7.946539060795657, \"y\": -3.3676865471908224, \"cluster\": 1}, {\"x\": -8.245985356429562, \"y\": -8.613158209058627, \"cluster\": 2}, {\"x\": -1.9819771099620271, \"y\": 4.022435514174746, \"cluster\": 1}, {\"x\": -4.350980346475657, \"y\": -3.6947667796776424, \"cluster\": 1}, {\"x\": -10.476869602439182, \"y\": -3.6031813885349258, \"cluster\": 2}, {\"x\": -11.01959844829441, \"y\": -3.1588203077621224, \"cluster\": 2}, {\"x\": -5.17255903986124, \"y\": -4.318359712685473, \"cluster\": 1}, {\"x\": -2.406718199699357, \"y\": 6.098944469870908, \"cluster\": 1}, {\"x\": -6.721494975923703, \"y\": -2.8844080642178818, \"cluster\": 1}, {\"x\": -6.58935963436387, \"y\": -4.43379547940742, \"cluster\": 1}, {\"x\": -1.4612601858217498, \"y\": 4.525498514650621, \"cluster\": 1}, {\"x\": -0.9190034548522448, \"y\": 3.452789267035227, \"cluster\": 1}, {\"x\": -10.409351666841584, \"y\": -2.674820464521483, \"cluster\": 2}, {\"x\": -6.367228090988169, \"y\": -3.326660724595877, \"cluster\": 1}, {\"x\": -6.72766124534928, \"y\": -7.145162671548282, \"cluster\": 1}, {\"x\": -2.279560749896649, \"y\": 5.104521902915381, \"cluster\": 1}, {\"x\": -5.848875595943767, \"y\": -3.0397050619974144, \"cluster\": 1}, {\"x\": -6.079930511938528, \"y\": -7.081975676778154, \"cluster\": 1}, {\"x\": -5.266829294846004, \"y\": -2.696450548014737, \"cluster\": 1}, {\"x\": -6.05367511988283, \"y\": -9.629790765144225, \"cluster\": 0}, {\"x\": -10.08222053105773, \"y\": -4.250710430136708, \"cluster\": 2}, {\"x\": -11.870873481288402, \"y\": -3.0327334293082484, \"cluster\": 2}, {\"x\": -5.371073070369658, \"y\": -7.9563583253737535, \"cluster\": 1}, {\"x\": -9.375908999347528, \"y\": -4.553153078434052, \"cluster\": 2}, {\"x\": -6.6340198681573215, \"y\": -2.583403556386127, \"cluster\": 1}, {\"x\": -9.54609655309786, \"y\": -2.8491742210599913, \"cluster\": 2}, {\"x\": -1.6982554152145706, \"y\": 2.7907175141398666, \"cluster\": 1}, {\"x\": -5.602176024325104, \"y\": -0.6599084896160656, \"cluster\": 1}, {\"x\": -6.03429022029922, \"y\": -4.088211960497224, \"cluster\": 1}, {\"x\": -6.372307836118383, \"y\": -8.631900461039706, \"cluster\": 1}, {\"x\": -10.226478332523264, \"y\": -2.339987172800089, \"cluster\": 2}, {\"x\": -5.956781476027413, \"y\": -3.979057011247294, \"cluster\": 1}, {\"x\": -1.4270653467697816, \"y\": 5.089041275707647, \"cluster\": 1}, {\"x\": -0.6207353040891055, \"y\": 6.59346951581742, \"cluster\": 1}, {\"x\": -0.32810279336092774, \"y\": 4.119182005367143, \"cluster\": 1}, {\"x\": -10.62305450511909, \"y\": -4.547191614617715, \"cluster\": 2}, {\"x\": -9.12674270112086, \"y\": -4.461805681638605, \"cluster\": 2}, {\"x\": -5.241344969182345, \"y\": -3.2350587337503987, \"cluster\": 1}, {\"x\": -7.199675312953462, \"y\": -7.104009813871477, \"cluster\": 1}, {\"x\": -10.113697689125498, \"y\": -4.128807516024957, \"cluster\": 2}, {\"x\": -10.341613212863495, \"y\": -4.95351773723572, \"cluster\": 2}, {\"x\": -12.504153156329163, \"y\": -6.067512469280031, \"cluster\": 2}, {\"x\": -9.323316398832233, \"y\": -4.675740452777355, \"cluster\": 2}, {\"x\": -7.320330019823476, \"y\": -2.733500952132815, \"cluster\": 1}, {\"x\": -2.205334073931087, \"y\": 4.207652005954265, \"cluster\": 1}, {\"x\": -0.5279305184970926, \"y\": 5.92630668526536, \"cluster\": 1}, {\"x\": -8.874300341631166, \"y\": -3.648081507119194, \"cluster\": 2}, {\"x\": -6.66948545344214, \"y\": -4.260598838360167, \"cluster\": 1}, {\"x\": -1.3739725806942609, \"y\": 5.291631033113889, \"cluster\": 1}, {\"x\": -6.600857078206719, \"y\": -3.1196968767431805, \"cluster\": 1}, {\"x\": -7.991754122866363, \"y\": -8.335648510987143, \"cluster\": 2}, {\"x\": -6.224478686299447, \"y\": -2.438462244886699, \"cluster\": 1}, {\"x\": -11.105425005937654, \"y\": -3.9710668652733467, \"cluster\": 2}, {\"x\": -8.957623350499006, \"y\": -4.87178858566323, \"cluster\": 2}, {\"x\": -6.654616438260894, \"y\": -7.293357132827133, \"cluster\": 1}, {\"x\": -10.95313776602679, \"y\": -3.367438116260589, \"cluster\": 2}, {\"x\": -5.113510079104775, \"y\": -2.0188199201889114, \"cluster\": 1}, {\"x\": -7.24251437964129, \"y\": -9.663684479711614, \"cluster\": 2}, {\"x\": -5.349294557744197, \"y\": -3.545773316732582, \"cluster\": 1}, {\"x\": -6.2926133215005216, \"y\": -3.6889242583746578, \"cluster\": 1}, {\"x\": -7.353879526667555, \"y\": -8.545044344432092, \"cluster\": 2}, {\"x\": -5.784234728636943, \"y\": -4.484068484204371, \"cluster\": 1}, {\"x\": -5.662563249318529, \"y\": -2.343900922698393, \"cluster\": 1}, {\"x\": -8.608933107282834, \"y\": -4.614692790516223, \"cluster\": 2}, {\"x\": -0.25201990570729094, \"y\": 4.535591448553887, \"cluster\": 1}, {\"x\": -1.9274479855745354, \"y\": 4.9368453355813475, \"cluster\": 1}, {\"x\": -9.413065893175675, \"y\": -3.6290743039147837, \"cluster\": 2}, {\"x\": -0.8486082331449638, \"y\": 5.4509319635504205, \"cluster\": 1}, {\"x\": -0.8667530400291719, \"y\": 3.7829591391633697, \"cluster\": 1}, {\"x\": -10.184291497164567, \"y\": -4.010173028249052, \"cluster\": 2}, {\"x\": -7.931929183434267, \"y\": -5.424505473195829, \"cluster\": 1}, {\"x\": -2.7544717516895614, \"y\": 4.575872301901829, \"cluster\": 1}, {\"x\": -11.717106978272625, \"y\": -3.8962275512811124, \"cluster\": 2}, {\"x\": -8.850812127253619, \"y\": -4.00305112652188, \"cluster\": 2}, {\"x\": -1.3439249587069142, \"y\": 2.3842886530191585, \"cluster\": 1}, {\"x\": -8.1620365439131, \"y\": -7.314593357395667, \"cluster\": 2}, {\"x\": -9.188868143387419, \"y\": -2.163593864156933, \"cluster\": 1}, {\"x\": -7.132292597186354, \"y\": -4.022967302286994, \"cluster\": 1}, {\"x\": -0.04261030709121938, \"y\": 4.909230750843066, \"cluster\": 1}, {\"x\": -7.244494478309097, \"y\": -7.65150299527888, \"cluster\": 1}, {\"x\": -8.137846462369383, \"y\": -7.658069493423728, \"cluster\": 2}, {\"x\": -6.7345134534846505, \"y\": -1.3833019413836023, \"cluster\": 1}, {\"x\": -8.963694242977034, \"y\": -9.270338797063157, \"cluster\": 2}, {\"x\": 0.8687658008578785, \"y\": 4.1578550911277015, \"cluster\": 1}, {\"x\": -0.5451769291687276, \"y\": 3.819965930021569, \"cluster\": 1}, {\"x\": -8.016944284363998, \"y\": -8.6713736570228, \"cluster\": 2}, {\"x\": -3.3337557135674514, \"y\": 5.231519693282148, \"cluster\": 1}, {\"x\": -11.43858852794494, \"y\": -2.7210954765347686, \"cluster\": 2}, {\"x\": -2.5208762667245614, \"y\": 5.081201394531134, \"cluster\": 1}, {\"x\": -6.8439444259585525, \"y\": -4.150582220449893, \"cluster\": 1}, {\"x\": -5.876197384285145, \"y\": -3.2807891557700892, \"cluster\": 1}, {\"x\": -1.2181954623799376, \"y\": 4.3063346355596615, \"cluster\": 1}, {\"x\": -2.0034135815195953, \"y\": 4.450086725677409, \"cluster\": 1}, {\"x\": -10.107703993816708, \"y\": -3.94479960192718, \"cluster\": 2}, {\"x\": -7.030458537548547, \"y\": -1.237347564565535, \"cluster\": 1}, {\"x\": -6.9568513663453935, \"y\": -8.12381048565328, \"cluster\": 1}, {\"x\": -2.3302221922375503, \"y\": 4.784053655164081, \"cluster\": 1}, {\"x\": -9.984359827305925, \"y\": -4.648042142650431, \"cluster\": 2}, {\"x\": -2.3308060367853387, \"y\": 4.39382526992426, \"cluster\": 1}, {\"x\": -10.779624186339902, \"y\": -4.390857530191786, \"cluster\": 2}, {\"x\": -2.034844856038634, \"y\": 3.7677594613889394, \"cluster\": 1}, {\"x\": -7.167442445959466, \"y\": -3.249983778680282, \"cluster\": 1}, {\"x\": -0.49922133595474993, \"y\": 4.775982585218885, \"cluster\": 1}, {\"x\": -5.766811435831203, \"y\": -3.4128177881590513, \"cluster\": 1}, {\"x\": -10.699056929910277, \"y\": -4.490571571238543, \"cluster\": 2}, {\"x\": -7.2872962121876474, \"y\": -6.683067760798305, \"cluster\": 1}, {\"x\": -8.178318286630729, \"y\": -8.220638129570899, \"cluster\": 2}, {\"x\": -9.144431281697479, \"y\": -4.366377857074236, \"cluster\": 2}, {\"x\": -7.223235428900379, \"y\": -3.5122637563645807, \"cluster\": 1}, {\"x\": -9.712964392804043, \"y\": -3.690881101899936, \"cluster\": 2}, {\"x\": -0.03190915279961315, \"y\": 4.744501565417638, \"cluster\": 1}, {\"x\": -7.104060439867236, \"y\": -8.381982275844257, \"cluster\": 1}, {\"x\": -7.5248250144984254, \"y\": -7.508874438593723, \"cluster\": 2}, {\"x\": -6.31161342896361, \"y\": -2.9764169703468255, \"cluster\": 1}, {\"x\": -0.5381421977128555, \"y\": 4.815390406779989, \"cluster\": 1}, {\"x\": -9.580410498678239, \"y\": -3.168577896347615, \"cluster\": 2}, {\"x\": -9.53106923642426, \"y\": -2.9196616794237036, \"cluster\": 2}, {\"x\": -10.765022329984836, \"y\": -3.278777840407201, \"cluster\": 2}, {\"x\": -9.546589564918502, \"y\": -4.648269448548444, \"cluster\": 2}, {\"x\": -7.3939337329988195, \"y\": -6.806122639953547, \"cluster\": 1}, {\"x\": -2.9915115724657677, \"y\": 2.645801308444378, \"cluster\": 1}, {\"x\": -5.675582536987324, \"y\": -4.559022548160358, \"cluster\": 1}, {\"x\": -3.5175417703952716, \"y\": 5.642653899295982, \"cluster\": 1}, {\"x\": -0.9985396183498266, \"y\": 6.198648077818729, \"cluster\": 1}, {\"x\": -5.964979013839094, \"y\": -2.0374646912902934, \"cluster\": 1}, {\"x\": -8.852795074857454, \"y\": -7.791380788784477, \"cluster\": 2}, {\"x\": -4.643104258815785, \"y\": -2.227894218120615, \"cluster\": 1}, {\"x\": -1.359389585992692, \"y\": 4.054240022349643, \"cluster\": 1}, {\"x\": -0.5257904636130821, \"y\": 3.3065986015291307, \"cluster\": 1}, {\"x\": -1.1563750925131935, \"y\": 5.6997157513754235, \"cluster\": 1}, {\"x\": -6.425300098097034, \"y\": -2.173286191076472, \"cluster\": 1}, {\"x\": -5.701833046489826, \"y\": -2.630838382829303, \"cluster\": 1}, {\"x\": -6.046329708889857, \"y\": -6.92266990466932, \"cluster\": 1}, {\"x\": -8.145592876103246, \"y\": -7.42775410014251, \"cluster\": 2}, {\"x\": -9.156850945511998, \"y\": -4.056235764936742, \"cluster\": 2}, {\"x\": -9.161707784218832, \"y\": -2.409989436558721, \"cluster\": 1}, {\"x\": -1.4686444212810534, \"y\": 6.506745005322004, \"cluster\": 1}, {\"x\": -6.746727983979049, \"y\": -8.172459738721683, \"cluster\": 1}, {\"x\": -1.9860594040267037, \"y\": 3.0638140794657254, \"cluster\": 1}, {\"x\": -10.32899567358573, \"y\": -3.5668094022301133, \"cluster\": 2}, {\"x\": -9.343132351066401, \"y\": -4.004536994970869, \"cluster\": 2}, {\"x\": -9.559546156361863, \"y\": -2.831020225706112, \"cluster\": 2}, {\"x\": -10.165911343397818, \"y\": -4.127528891671195, \"cluster\": 2}, {\"x\": -9.841448653383019, \"y\": -4.143569572448066, \"cluster\": 2}, {\"x\": -10.276810225196392, \"y\": -2.330499461409204, \"cluster\": 2}, {\"x\": -10.103057216160643, \"y\": -3.323152876678294, \"cluster\": 2}, {\"x\": -9.902287416461844, \"y\": -3.031898478703693, \"cluster\": 2}, {\"x\": -9.721213198830919, \"y\": -4.686620151258517, \"cluster\": 2}, {\"x\": -1.8513954583101344, \"y\": 3.5188609047583252, \"cluster\": 1}, {\"x\": -6.6932118924458734, \"y\": -6.300218619554666, \"cluster\": 1}, {\"x\": -6.533718390201513, \"y\": -8.149227262648669, \"cluster\": 1}, {\"x\": -8.463695002324215, \"y\": -8.071460285818429, \"cluster\": 2}, {\"x\": -5.750045280149759, \"y\": -3.565909665608377, \"cluster\": 1}, {\"x\": -1.1710417594110225, \"y\": 4.330918155822106, \"cluster\": 1}, {\"x\": -8.526285788379845, \"y\": -8.669576013270067, \"cluster\": 2}, {\"x\": -9.23890684355517, \"y\": -3.0684397328983213, \"cluster\": 2}, {\"x\": -6.128030509059693, \"y\": -2.516980580274055, \"cluster\": 1}, {\"x\": -8.10406450569264, \"y\": -7.420204872904966, \"cluster\": 2}, {\"x\": -1.6158909127701304, \"y\": 4.180175626329556, \"cluster\": 1}, {\"x\": -8.987585326318378, \"y\": -3.0333306141154415, \"cluster\": 1}, {\"x\": -11.941035909321455, \"y\": -3.6008541829938716, \"cluster\": 2}, {\"x\": -10.439941783166274, \"y\": -3.6298211937617726, \"cluster\": 2}, {\"x\": -11.424267924173634, \"y\": -2.185388599052178, \"cluster\": 2}, {\"x\": -9.009929142269778, \"y\": -9.068652472855751, \"cluster\": 2}, {\"x\": -6.474356490442799, \"y\": -3.7433886276640225, \"cluster\": 1}, {\"x\": -9.631380489113043, \"y\": -4.9979379292709964, \"cluster\": 2}, {\"x\": 0.5260155005846419, \"y\": 3.009993533355024, \"cluster\": 1}, {\"x\": -9.763243925323785, \"y\": -9.366566230452749, \"cluster\": 2}, {\"x\": -6.2796552640230505, \"y\": -8.818095871591257, \"cluster\": 1}, {\"x\": -9.468832758067927, \"y\": -6.190435058475675, \"cluster\": 2}, {\"x\": -5.773366182553252, \"y\": -3.5673995330002293, \"cluster\": 1}, {\"x\": -6.692425332146296, \"y\": -8.301717907683438, \"cluster\": 1}, {\"x\": -7.444399699191618, \"y\": -9.168031795482715, \"cluster\": 2}, {\"x\": -7.11478468705883, \"y\": -5.3869913396768965, \"cluster\": 1}, {\"x\": -0.3858039757897429, \"y\": 6.373591618097896, \"cluster\": 1}, {\"x\": -2.004547116103499, \"y\": 4.175650125707692, \"cluster\": 1}, {\"x\": -5.755176277975085, \"y\": -9.30821073955341, \"cluster\": 0}, {\"x\": -9.141684205581292, \"y\": -7.205726937278369, \"cluster\": 2}, {\"x\": -5.920925345563742, \"y\": -3.2757404756717943, \"cluster\": 1}, {\"x\": -2.351220657673829, \"y\": 4.0097363419871845, \"cluster\": 1}, {\"x\": -5.919078510870565, \"y\": -2.2391986140442226, \"cluster\": 1}, {\"x\": -5.6220052551614295, \"y\": -8.692909666826537, \"cluster\": 1}, {\"x\": -7.54246303831446, \"y\": -8.127228110438386, \"cluster\": 2}, {\"x\": -2.4139578469451726, \"y\": 5.659358024076449, \"cluster\": 1}, {\"x\": -6.371515957997354, \"y\": -8.911295433180277, \"cluster\": 1}, {\"x\": -12.14017923273047, \"y\": -4.783517411385913, \"cluster\": 2}, {\"x\": -4.45264490609506, \"y\": 6.344018682459241, \"cluster\": 1}, {\"x\": -5.596988201326771, \"y\": -4.19535852653093, \"cluster\": 1}, {\"x\": -6.075036217253955, \"y\": -2.156064045450953, \"cluster\": 1}, {\"x\": -7.248282380977763, \"y\": -7.052227901955563, \"cluster\": 1}, {\"x\": -4.778911006562106, \"y\": -2.4133316499348325, \"cluster\": 1}, {\"x\": -12.411215504826888, \"y\": -5.730914923169192, \"cluster\": 2}, {\"x\": -6.7526434946447385, \"y\": -8.346549745098049, \"cluster\": 1}, {\"x\": -5.054921393415961, \"y\": -4.222577494532622, \"cluster\": 1}, {\"x\": -10.382544752485892, \"y\": -2.4952403087536847, \"cluster\": 2}, {\"x\": -7.225705023497352, \"y\": -3.7931357906625034, \"cluster\": 1}, {\"x\": -11.949817790898333, \"y\": -5.355677692816341, \"cluster\": 2}, {\"x\": -7.628670918240042, \"y\": -8.063541697301455, \"cluster\": 2}, {\"x\": -4.617671134403657, \"y\": -1.6711114461396843, \"cluster\": 1}, {\"x\": -5.122196637435978, \"y\": -3.3130212312625433, \"cluster\": 1}, {\"x\": -6.29225072428276, \"y\": -2.357382942013979, \"cluster\": 1}, {\"x\": 0.00024227116135100424, \"y\": 5.148534029420497, \"cluster\": 1}, {\"x\": -8.79988166193185, \"y\": -2.2487543759907975, \"cluster\": 1}, {\"x\": -2.7768702545837973, \"y\": 4.640905566660254, \"cluster\": 1}, {\"x\": -6.396949787558266, \"y\": -3.769637033156362, \"cluster\": 1}, {\"x\": -6.9226308111314205, \"y\": -7.639722624635498, \"cluster\": 1}, {\"x\": -11.576868789047566, \"y\": -4.78197652621961, \"cluster\": 2}, {\"x\": -5.668247372033288, \"y\": -3.8260750868989764, \"cluster\": 1}, {\"x\": -11.157882608635378, \"y\": -2.6032417286814784, \"cluster\": 2}, {\"x\": -10.473085378791083, \"y\": -3.4757383655456473, \"cluster\": 2}, {\"x\": -9.981184936347486, \"y\": -3.776160827087244, \"cluster\": 2}, {\"x\": -1.0410207751622265, \"y\": 3.9633179381425236, \"cluster\": 1}, {\"x\": -9.328560152872887, \"y\": -2.6089330879852275, \"cluster\": 2}, {\"x\": -1.1389835688751488, \"y\": 3.262148479220019, \"cluster\": 1}, {\"x\": -6.179056384691831, \"y\": -7.9633664552409495, \"cluster\": 1}, {\"x\": -10.235654439405284, \"y\": -2.798060661274979, \"cluster\": 2}, {\"x\": -5.771332562994431, \"y\": -8.592225768290323, \"cluster\": 1}, {\"x\": -9.145008441927814, \"y\": -3.9179884503084472, \"cluster\": 2}, {\"x\": -1.6173461592329268, \"y\": 4.9893050825589835, \"cluster\": 1}, {\"x\": -2.7786753045082477, \"y\": 6.362568772546803, \"cluster\": 1}, {\"x\": -9.54642848761293, \"y\": -5.637408533231442, \"cluster\": 2}, {\"x\": -6.914865904929935, \"y\": -7.6896937828923795, \"cluster\": 1}, {\"x\": -1.8461296778558682, \"y\": 4.304743996314016, \"cluster\": 1}, {\"x\": -5.5283458556366405, \"y\": -8.1536031133824, \"cluster\": 1}, {\"x\": -6.009153369932298, \"y\": -3.3492515180044244, \"cluster\": 1}, {\"x\": -8.546283243062167, \"y\": -4.571385395044784, \"cluster\": 2}, {\"x\": -7.316556391289163, \"y\": -7.770512930899681, \"cluster\": 1}, {\"x\": -7.20423398910866, \"y\": -8.881765594048746, \"cluster\": 2}, {\"x\": -7.556007321448557, \"y\": -8.018854988584291, \"cluster\": 2}, {\"x\": -5.67856791941142, \"y\": -7.605098519935852, \"cluster\": 1}, {\"x\": -5.214468263289775, \"y\": -4.799953118763537, \"cluster\": 1}, {\"x\": -9.376629802814094, \"y\": -2.9972268432385083, \"cluster\": 2}, {\"x\": -5.318447094429766, \"y\": -8.92829839169748, \"cluster\": 1}, {\"x\": -10.827884398968216, \"y\": -4.833926147207375, \"cluster\": 2}, {\"x\": -6.065699095871062, \"y\": -1.5337694648059559, \"cluster\": 1}, {\"x\": -2.346732606068119, \"y\": 3.561284227344442, \"cluster\": 1}, {\"x\": -1.2560682641577197, \"y\": 5.0000683920802285, \"cluster\": 1}, {\"x\": -5.839797448183742, \"y\": -2.178361856135961, \"cluster\": 1}, {\"x\": -6.8708821070342445, \"y\": -2.227162357797841, \"cluster\": 1}, {\"x\": -1.7960046498445525, \"y\": 4.287435681068351, \"cluster\": 1}, {\"x\": -9.379726969732767, \"y\": -4.13752487301695, \"cluster\": 2}, {\"x\": -7.236059371477093, \"y\": -4.547109919532199, \"cluster\": 1}, {\"x\": -10.279448772718714, \"y\": -1.8969930241649111, \"cluster\": 2}, {\"x\": -1.4168904648667249, \"y\": 4.608320047717165, \"cluster\": 1}, {\"x\": -5.780454121464204, \"y\": -4.582979224413729, \"cluster\": 1}, {\"x\": 0.08525185826796045, \"y\": 3.6452829679480585, \"cluster\": 1}, {\"x\": -9.202686409228257, \"y\": -4.327786866206425, \"cluster\": 2}, {\"x\": -9.568186363631137, \"y\": -4.560346945563251, \"cluster\": 2}, {\"x\": -1.1643485819705592, \"y\": 4.2317867091006605, \"cluster\": 1}, {\"x\": -6.163458506063613, \"y\": -3.108308017459318, \"cluster\": 1}, {\"x\": -6.321525640143877, \"y\": -9.662800788848088, \"cluster\": 0}, {\"x\": -7.520999738806424, \"y\": -9.133118356285845, \"cluster\": 2}, {\"x\": -9.220293298404238, \"y\": -4.0721197194662935, \"cluster\": 2}, {\"x\": -10.849168160183929, \"y\": -2.9524671236829034, \"cluster\": 2}, {\"x\": -9.863664307098478, \"y\": -2.751293685163499, \"cluster\": 2}, {\"x\": -6.797152237582278, \"y\": -3.458041360968013, \"cluster\": 1}, {\"x\": -0.979490066070415, \"y\": 4.08668826994449, \"cluster\": 1}, {\"x\": -2.060438097737446, \"y\": 5.230495487293569, \"cluster\": 1}, {\"x\": -5.668391828142745, \"y\": -0.7950678467818668, \"cluster\": 1}, {\"x\": -0.757969185355724, \"y\": 4.908984207745029, \"cluster\": 1}, {\"x\": -10.420569465043878, \"y\": -3.866884140838917, \"cluster\": 2}, {\"x\": -7.124250086206238, \"y\": -6.704238700465809, \"cluster\": 1}, {\"x\": -1.3788948288258804, \"y\": 4.333377165095884, \"cluster\": 1}, {\"x\": -6.614664441619893, \"y\": -7.525791021870934, \"cluster\": 1}, {\"x\": -1.340520809891421, \"y\": 4.157119493365752, \"cluster\": 1}, {\"x\": -6.211599997834005, \"y\": -8.292939835065118, \"cluster\": 1}, {\"x\": -7.5688561328000565, \"y\": -8.135272212973218, \"cluster\": 2}, {\"x\": -1.7700069323658358, \"y\": 3.789127805130801, \"cluster\": 1}, {\"x\": -7.365858337613077, \"y\": -7.345772187618775, \"cluster\": 1}, {\"x\": -1.499522836500689, \"y\": 5.282658789959386, \"cluster\": 1}, {\"x\": -2.8588279382837056, \"y\": 5.2698351863871835, \"cluster\": 1}, {\"x\": -7.738849346027174, \"y\": -3.2432766529495733, \"cluster\": 1}, {\"x\": -10.820179692557353, \"y\": -3.231637255250512, \"cluster\": 2}, {\"x\": -8.536820123190024, \"y\": -3.3608757459391847, \"cluster\": 1}, {\"x\": -12.034913729333896, \"y\": -5.895937728839661, \"cluster\": 2}, {\"x\": -5.269109092135366, \"y\": -2.7352182431364156, \"cluster\": 1}, {\"x\": -6.712996039785251, \"y\": -2.9032498381724894, \"cluster\": 1}, {\"x\": -8.361186337903108, \"y\": -2.726983818567998, \"cluster\": 1}, {\"x\": -5.489414275085358, \"y\": -6.946620206588477, \"cluster\": 1}, {\"x\": 0.5311398230212139, \"y\": 2.510128945952069, \"cluster\": 1}, {\"x\": -5.6412677543087515, \"y\": -7.249228930620518, \"cluster\": 1}, {\"x\": -9.482638890013735, \"y\": -6.735883015016127, \"cluster\": 2}, {\"x\": -7.531037040483667, \"y\": -6.768236760975406, \"cluster\": 1}, {\"x\": -6.310785954173774, \"y\": -2.051746477365137, \"cluster\": 1}, {\"x\": -8.70233177666608, \"y\": -4.194625400586082, \"cluster\": 2}, {\"x\": -6.110130706597376, \"y\": -2.3106112800394785, \"cluster\": 1}, {\"x\": -5.839726334384508, \"y\": -9.206774178324359, \"cluster\": 0}, {\"x\": -11.753638143885953, \"y\": -3.2385589498773895, \"cluster\": 2}, {\"x\": -9.291994815507753, \"y\": -9.852561709346341, \"cluster\": 2}, {\"x\": -7.855682137432907, \"y\": -6.9295058926639355, \"cluster\": 2}, {\"x\": -10.196710687456825, \"y\": -2.0868771722110795, \"cluster\": 2}, {\"x\": -7.963565376976385, \"y\": -7.833571162679451, \"cluster\": 2}, {\"x\": -6.776804016652588, \"y\": -6.655119921047011, \"cluster\": 1}, {\"x\": -10.87499402257129, \"y\": -4.82113577023613, \"cluster\": 2}, {\"x\": -1.84048020802667, \"y\": 3.802569241049904, \"cluster\": 1}, {\"x\": -7.980674032726418, \"y\": -8.56048014764401, \"cluster\": 2}, {\"x\": -6.32066245851635, \"y\": -3.3075189181067537, \"cluster\": 1}, {\"x\": -6.179799662377421, \"y\": -3.0080344705160247, \"cluster\": 1}, {\"x\": -2.1766543638687477, \"y\": 3.4094630411928986, \"cluster\": 1}, {\"x\": -0.6732247181272776, \"y\": 4.620023770178603, \"cluster\": 1}, {\"x\": -8.938921711708721, \"y\": -3.5152140767725673, \"cluster\": 2}, {\"x\": -7.489374974228666, \"y\": -8.884759086354379, \"cluster\": 2}, {\"x\": -2.8964132824899167, \"y\": 5.282328796492461, \"cluster\": 1}, {\"x\": -0.8133992583634861, \"y\": 3.546973928011175, \"cluster\": 1}, {\"x\": -5.7775266738006055, \"y\": -2.851452759799448, \"cluster\": 1}, {\"x\": -6.248838499333678, \"y\": -8.765635077752869, \"cluster\": 1}, {\"x\": -3.103673711378109, \"y\": 3.90202400589671, \"cluster\": 1}, {\"x\": -1.0572406256528906, \"y\": 4.826772072490221, \"cluster\": 1}, {\"x\": -5.732150477880266, \"y\": -5.046954544738055, \"cluster\": 1}, {\"x\": -9.936962307853594, \"y\": -3.742223792355488, \"cluster\": 2}, {\"x\": -3.0326772261952755, \"y\": 4.721649260885453, \"cluster\": 1}, {\"x\": -10.703553011055385, \"y\": -2.760662479608511, \"cluster\": 2}, {\"x\": -5.684756308285753, \"y\": -3.768169242136279, \"cluster\": 1}, {\"x\": -8.62182374152277, \"y\": -8.76567023290839, \"cluster\": 2}, {\"x\": -6.671772938803755, \"y\": -9.977147957249239, \"cluster\": 0}, {\"x\": -1.9257784119521415, \"y\": 4.43910441553652, \"cluster\": 1}, {\"x\": -8.162994877003415, \"y\": -3.388965691868891, \"cluster\": 1}, {\"x\": -3.743803427487374, \"y\": -8.75345343500136, \"cluster\": 1}, {\"x\": -5.666012113538473, \"y\": -4.970196326305104, \"cluster\": 1}, {\"x\": -2.8896180416154813, \"y\": 4.957027364819377, \"cluster\": 1}, {\"x\": -2.3599584108697744, \"y\": 4.203095419878603, \"cluster\": 1}, {\"x\": -6.8049155726466335, \"y\": -3.4960254831509756, \"cluster\": 1}, {\"x\": -7.104806758014256, \"y\": -4.108305308544761, \"cluster\": 1}, {\"x\": -6.9668553912810705, \"y\": -3.128763922329747, \"cluster\": 1}, {\"x\": -6.313544948497254, \"y\": -8.012832668176694, \"cluster\": 1}, {\"x\": -4.471206790960085, \"y\": -3.5413104324230704, \"cluster\": 1}, {\"x\": -1.5394009534668904, \"y\": 5.023692978550581, \"cluster\": 1}, {\"x\": -1.6087521511724905, \"y\": 3.769494222273808, \"cluster\": 1}, {\"x\": -10.192769844529003, \"y\": -3.147955123131023, \"cluster\": 2}, {\"x\": -2.8020781039706595, \"y\": 4.057147146430284, \"cluster\": 1}, {\"x\": 0.24509880239246162, \"y\": 5.517546567403667, \"cluster\": 1}, {\"x\": -3.31028117177262, \"y\": 3.515934284680113, \"cluster\": 1}, {\"x\": -2.841878032458153, \"y\": 3.740735350644035, \"cluster\": 1}, {\"x\": -5.75867611852229, \"y\": -8.75783107378842, \"cluster\": 1}, {\"x\": -5.995910561456476, \"y\": -8.112856673601206, \"cluster\": 1}, {\"x\": -4.9836068695957065, \"y\": -3.2052296074107085, \"cluster\": 1}, {\"x\": -1.8684541393232976, \"y\": 4.993113060025359, \"cluster\": 1}, {\"x\": -9.715036791404682, \"y\": -4.779445979210525, \"cluster\": 2}, {\"x\": -6.473733222730479, \"y\": -2.7868254137329, \"cluster\": 1}, {\"x\": -6.992630284501854, \"y\": -7.143440769676486, \"cluster\": 1}, {\"x\": -1.5377386349570825, \"y\": 5.535973776755081, \"cluster\": 1}, {\"x\": -10.446450536324724, \"y\": -4.625796586149801, \"cluster\": 2}, {\"x\": -1.0967988093294934, \"y\": 4.647226961080894, \"cluster\": 1}, {\"x\": -7.252568773979549, \"y\": -2.9168283269714537, \"cluster\": 1}, {\"x\": -0.19745196890354544, \"y\": 2.3463491593455075, \"cluster\": 1}, {\"x\": -10.067041200343585, \"y\": -4.0617406145367445, \"cluster\": 2}, {\"x\": -6.13468589114183, \"y\": -4.507934236283532, \"cluster\": 1}, {\"x\": -10.372517190955755, \"y\": -4.703318164638871, \"cluster\": 2}, {\"x\": -1.8818880485588787, \"y\": 4.205731799913164, \"cluster\": 1}, {\"x\": -7.154984836988162, \"y\": -3.1077859806863612, \"cluster\": 1}, {\"x\": -6.142547993526798, \"y\": -3.652022063256501, \"cluster\": 1}, {\"x\": -7.427494271234997, \"y\": -9.6383845559495, \"cluster\": 2}, {\"x\": -1.1300945815958103, \"y\": 4.544191078840548, \"cluster\": 1}, {\"x\": -6.284855048319109, \"y\": -8.782669708677824, \"cluster\": 1}, {\"x\": -7.3332534877783155, \"y\": -8.284903730911063, \"cluster\": 2}, {\"x\": -6.403201107191756, \"y\": -7.166875924411981, \"cluster\": 1}, {\"x\": -7.2218758561510565, \"y\": -9.488430827639236, \"cluster\": 2}, {\"x\": -6.098342933428682, \"y\": -7.440179054320776, \"cluster\": 1}, {\"x\": -7.208077928664976, \"y\": -7.120244325674294, \"cluster\": 1}, {\"x\": -9.687440215869334, \"y\": -6.047596363586017, \"cluster\": 2}, {\"x\": -7.873729380129867, \"y\": -7.595788651766012, \"cluster\": 2}, {\"x\": -1.1466300855305107, \"y\": 4.108397033740446, \"cluster\": 1}, {\"x\": -5.903442201983345, \"y\": -8.180757490892024, \"cluster\": 1}, {\"x\": -2.760179083161441, \"y\": 5.551213578682775, \"cluster\": 1}, {\"x\": -1.2360655518844066, \"y\": 4.483829937191721, \"cluster\": 1}, {\"x\": -9.97584966702655, \"y\": -4.422022363641096, \"cluster\": 2}, {\"x\": -2.1066884707345177, \"y\": 5.63099757364866, \"cluster\": 1}, {\"x\": -4.73558876279152, \"y\": -4.237489688775991, \"cluster\": 1}, {\"x\": -10.723309622787529, \"y\": -4.821117224986795, \"cluster\": 2}, {\"x\": -8.26074369398222, \"y\": -5.647247819305372, \"cluster\": 2}, {\"x\": -6.883843439299868, \"y\": -7.046052653674982, \"cluster\": 1}, {\"x\": -2.1577734695795976, \"y\": 4.095504890540311, \"cluster\": 1}, {\"x\": -7.859884436213619, \"y\": -4.73888254428344, \"cluster\": 1}, {\"x\": -0.46064202604701254, \"y\": 4.591646286327105, \"cluster\": 1}, {\"x\": -5.0568548716452355, \"y\": -5.029466423503938, \"cluster\": 1}, {\"x\": -7.660550064568029, \"y\": -8.462349423265907, \"cluster\": 2}, {\"x\": -8.419239819175033, \"y\": -3.458347884476478, \"cluster\": 1}, {\"x\": -10.994732299182784, \"y\": -4.06014253414243, \"cluster\": 2}, {\"x\": -6.7137652871467735, \"y\": -8.221998567702684, \"cluster\": 1}, {\"x\": -10.797259994344861, \"y\": -4.244943142864037, \"cluster\": 2}, {\"x\": -8.237463277204007, \"y\": -4.014001039138014, \"cluster\": 1}, {\"x\": -2.932118661194514, \"y\": 4.720037589306593, \"cluster\": 1}, {\"x\": -1.6614513887865232, \"y\": 3.0098694442477294, \"cluster\": 1}, {\"x\": -7.657343468811008, \"y\": -10.458136042394184, \"cluster\": 2}, {\"x\": -9.980547777279012, \"y\": -4.382490825598298, \"cluster\": 2}, {\"x\": -5.519403743255727, \"y\": -2.3878033376158005, \"cluster\": 1}, {\"x\": -1.9696766794665796, \"y\": 1.9716521047360227, \"cluster\": 1}, {\"x\": -3.884649810113071, \"y\": -2.843362609949925, \"cluster\": 1}, {\"x\": -5.82969906210201, \"y\": -2.9906732129170535, \"cluster\": 1}, {\"x\": -6.667001758775887, \"y\": -9.149238993712444, \"cluster\": 0}, {\"x\": -6.628895987361329, \"y\": -8.840715497684219, \"cluster\": 1}, {\"x\": -6.489449614672605, \"y\": -2.0675373330401436, \"cluster\": 1}, {\"x\": -7.171342305213279, \"y\": -10.944224511317891, \"cluster\": 2}, {\"x\": -11.30424657649712, \"y\": -3.8769680672036126, \"cluster\": 2}, {\"x\": -9.536548403676084, \"y\": -5.12933121880452, \"cluster\": 2}, {\"x\": -6.098661321079398, \"y\": -7.427311252103224, \"cluster\": 1}, {\"x\": -8.789256175101139, \"y\": -2.837646744578718, \"cluster\": 1}, {\"x\": -7.323865037058271, \"y\": -7.963934907065387, \"cluster\": 1}, {\"x\": -10.033080428531813, \"y\": -1.842743493762495, \"cluster\": 2}, {\"x\": -1.0361977293704867, \"y\": 3.971533185887934, \"cluster\": 1}, {\"x\": -6.428298774243108, \"y\": -6.743974715260037, \"cluster\": 1}, {\"x\": -2.8793043029275522, \"y\": 6.855858517904559, \"cluster\": 1}, {\"x\": -10.52994652455128, \"y\": -2.835215150645509, \"cluster\": 2}, {\"x\": -6.114230780704071, \"y\": -3.2089354347677936, \"cluster\": 1}, {\"x\": -1.7824501314671677, \"y\": 3.4707204345840927, \"cluster\": 1}, {\"x\": -8.952718093979204, \"y\": -3.344833848878355, \"cluster\": 2}, {\"x\": -5.166179014456907, \"y\": -3.7917058620492132, \"cluster\": 1}, {\"x\": -1.6421504976485148, \"y\": 3.284471140096273, \"cluster\": 1}, {\"x\": -8.335342960043219, \"y\": -7.870232570813044, \"cluster\": 2}, {\"x\": -6.311077061000285, \"y\": -3.921180806827734, \"cluster\": 1}, {\"x\": -1.7800244777370269, \"y\": 3.1733691333967347, \"cluster\": 1}, {\"x\": -1.6841768618242978, \"y\": 3.631328249674002, \"cluster\": 1}, {\"x\": -10.555207225439144, \"y\": -3.0141798031667166, \"cluster\": 2}, {\"x\": -5.343540093678463, \"y\": -2.138976643927715, \"cluster\": 1}, {\"x\": -11.536505749397744, \"y\": -4.401243732079323, \"cluster\": 2}, {\"x\": -4.895037580840365, \"y\": -2.486334557899435, \"cluster\": 1}, {\"x\": -5.443969898035991, \"y\": -8.959412922006269, \"cluster\": 1}, {\"x\": -1.5817387780358136, \"y\": 5.024870130841686, \"cluster\": 1}, {\"x\": -7.0299385942939985, \"y\": -6.699310523015227, \"cluster\": 1}, {\"x\": -6.170742382737493, \"y\": -2.5607820370672085, \"cluster\": 1}, {\"x\": -2.2218653369676096, \"y\": 6.361367943852196, \"cluster\": 1}, {\"x\": -7.5738544614806, \"y\": -8.319714059510003, \"cluster\": 2}, {\"x\": -7.658225937900157, \"y\": -7.642920507878789, \"cluster\": 2}, {\"x\": -6.895012928310167, \"y\": -9.317236075785248, \"cluster\": 0}, {\"x\": -11.114182521537845, \"y\": -3.872421446389926, \"cluster\": 2}, {\"x\": -0.794152276623841, \"y\": 2.104951171962879, \"cluster\": 1}, {\"x\": -6.428031934213674, \"y\": -5.521293971786157, \"cluster\": 1}, {\"x\": -5.897807017657774, \"y\": -8.192896804714527, \"cluster\": 1}, {\"x\": -6.5916969689749285, \"y\": -2.4477995927555383, \"cluster\": 1}, {\"x\": -6.457857759408556, \"y\": -3.3098143609273802, \"cluster\": 1}, {\"x\": -10.775571330280513, \"y\": -2.837507439439036, \"cluster\": 2}, {\"x\": -10.234149455466088, \"y\": -3.2255350473635556, \"cluster\": 2}, {\"x\": -6.266818388358796, \"y\": -8.255160143887645, \"cluster\": 1}, {\"x\": -5.205809800827664, \"y\": -3.2985383945615987, \"cluster\": 1}, {\"x\": -5.460452643297657, \"y\": -2.3083155293806703, \"cluster\": 1}, {\"x\": -7.0425995184711425, \"y\": -3.453323508845682, \"cluster\": 1}, {\"x\": -6.099628043228513, \"y\": -3.142269151874201, \"cluster\": 1}, {\"x\": -5.660069496619134, \"y\": -3.437769650054597, \"cluster\": 1}, {\"x\": -7.080973984400039, \"y\": -3.039723766765942, \"cluster\": 1}, {\"x\": -8.412647124911402, \"y\": -6.682488247932676, \"cluster\": 2}, {\"x\": -7.365134104901397, \"y\": -1.3885973124935187, \"cluster\": 1}, {\"x\": -10.416650401331225, \"y\": -4.4325334624906105, \"cluster\": 2}, {\"x\": -6.416238539567072, \"y\": -8.045884810800455, \"cluster\": 1}, {\"x\": -5.889193480102257, \"y\": -2.370494723384702, \"cluster\": 1}, {\"x\": -1.4294651705841361, \"y\": 5.168501049155187, \"cluster\": 1}, {\"x\": -6.561180687841756, \"y\": -3.959673108791006, \"cluster\": 1}, {\"x\": -1.4729985149602352, \"y\": 4.816541516051418, \"cluster\": 1}, {\"x\": -5.881008041197264, \"y\": -3.316926149839501, \"cluster\": 1}, {\"x\": -10.412559404941337, \"y\": -3.5014025100419817, \"cluster\": 2}, {\"x\": -8.552093769106948, \"y\": -3.158409998594174, \"cluster\": 1}, {\"x\": -0.7906737489426516, \"y\": 5.156901508708169, \"cluster\": 1}, {\"x\": -0.10075436532866022, \"y\": 4.515892565268589, \"cluster\": 1}, {\"x\": -1.3090139272844121, \"y\": 3.09420645760573, \"cluster\": 1}, {\"x\": -9.547556990885923, \"y\": -2.188013446844133, \"cluster\": 2}, {\"x\": -5.320300105898895, \"y\": -2.99303868802891, \"cluster\": 1}, {\"x\": -9.48229870186986, \"y\": -5.068219599729142, \"cluster\": 2}, {\"x\": -6.743616269173907, \"y\": -8.87844303036941, \"cluster\": 1}, {\"x\": -10.25189237126047, \"y\": -2.5535046048822188, \"cluster\": 2}, {\"x\": -1.9657639185768914, \"y\": 5.234464511450407, \"cluster\": 1}, {\"x\": -5.88036774243447, \"y\": -2.363262900885486, \"cluster\": 1}, {\"x\": -7.347745735292583, \"y\": -8.419554988942766, \"cluster\": 2}, {\"x\": -0.7587039566841077, \"y\": 3.7227620096688283, \"cluster\": 1}, {\"x\": -8.413578630454982, \"y\": -6.850692568137864, \"cluster\": 2}, {\"x\": -0.8205764920740146, \"y\": 5.337591950146718, \"cluster\": 1}, {\"x\": -7.934890406848591, \"y\": -7.784037635077171, \"cluster\": 2}, {\"x\": -5.694465659843353, \"y\": -4.062053044388285, \"cluster\": 1}, {\"x\": -0.8576988740771748, \"y\": 4.453057167267307, \"cluster\": 1}, {\"x\": 0.15097500817693676, \"y\": 3.100762946285424, \"cluster\": 1}, {\"x\": -6.553944409165649, \"y\": -6.44256626595291, \"cluster\": 1}, {\"x\": -10.931627159279603, \"y\": -4.4863688734468035, \"cluster\": 2}, {\"x\": -6.501555964616166, \"y\": -4.653293312950241, \"cluster\": 1}, {\"x\": -6.9365051935017945, \"y\": -6.392812921048723, \"cluster\": 1}, {\"x\": -10.13368982975369, \"y\": -4.750618332856334, \"cluster\": 2}, {\"x\": -9.891489779617883, \"y\": -5.479028863592562, \"cluster\": 2}, {\"x\": -8.898716170905923, \"y\": -4.854983037839203, \"cluster\": 2}, {\"x\": -8.113949928826354, \"y\": -7.836569209467391, \"cluster\": 2}, {\"x\": -5.290783535506342, \"y\": -3.6484668810576015, \"cluster\": 1}, {\"x\": -1.410760744560749, \"y\": 4.109848716472299, \"cluster\": 1}, {\"x\": -9.505375947716452, \"y\": -4.63402668837209, \"cluster\": 2}, {\"x\": -7.827494555623418, \"y\": -2.5103210388828026, \"cluster\": 1}, {\"x\": -6.380880855475786, \"y\": -8.506638087491059, \"cluster\": 1}, {\"x\": -8.960149131134205, \"y\": -8.063498987179393, \"cluster\": 2}, {\"x\": -7.666038984307122, \"y\": -7.59715459454666, \"cluster\": 2}, {\"x\": -6.465344070108034, \"y\": -2.855446332878923, \"cluster\": 1}], \"data-d937417af026e4f584a32531c2ef6637\": [{\"x\": -4, \"y\": -10}, {\"x\": -4, \"y\": -8}, {\"x\": -10, \"y\": -10}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.LayerChart(...)"
      ]
     },
     "execution_count": 7,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "c1 = alt.Chart(df_data).mark_circle().encode(\n",
    "    x=\"x\",\n",
    "    y=\"y\",\n",
    "    color=\"cluster:N\"\n",
    ")\n",
    "\n",
    "c2 = alt.Chart(df_centroids).mark_point(\n",
    "    size=500,\n",
    "    shape='cross',\n",
    "    filled=True,\n",
    "    stroke=\"black\",\n",
    ").encode(\n",
    "    x=\"x\",\n",
    "    y=\"y\",\n",
    ")\n",
    "\n",
    "c1+c2"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "I want to give some more explanation of where the `row.values` in the `apply` part came from.  Here is a reminder of what our original random data looked like.  The NumPy array `X` was produced by `make_blobs`."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "metadata": {
    "cell_id": "f3c9a2ea347f4b6ea8eec3a72dddac9c",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 4,
    "execution_start": 1667409113144,
    "source_hash": "aa4dcd2e",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "array([[ -6.92324165, -10.66953197],\n",
       "       [ -8.63062033,  -7.13940564],\n",
       "       [ -9.63048069,  -2.72044935],\n",
       "       [ -2.30647659,   5.30797676],\n",
       "       [ -7.57005366,  -3.01446491]])"
      ]
     },
     "execution_count": 12,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "X[:5]"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Here is a single point from among the 500 points.  It is a one-dimensional length-2 NumPy array."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "metadata": {
    "cell_id": "3b65186b00ca497a97fe8727496bbc30",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 4,
    "execution_start": 1667409120935,
    "source_hash": "dd0a0225",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "array([-10.4768696 ,  -3.60318139])"
      ]
     },
     "execution_count": 13,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "z = X[20] # NumPy version of df.iloc[20]\n",
    "z"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Here is the version of that point from our DataFrame, `df_data`."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 14,
   "metadata": {
    "cell_id": "11ffb43ee3b243aa90812fa8b0667e90",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 4,
    "execution_start": 1667409187217,
    "source_hash": "7ae6c44c",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "x         -10.476870\n",
       "y          -3.603181\n",
       "cluster     2.000000\n",
       "Name: 20, dtype: float64"
      ]
     },
     "execution_count": 14,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "df_data.iloc[20]"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Let's just look at the `\"x\"` and `\"y\"` portions of the pandas Series."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "metadata": {
    "cell_id": "068c8262fbe249a18bfca422f0d0b03b",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 4,
    "execution_start": 1667409209958,
    "source_hash": "d2e9a0d6",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "x   -10.476870\n",
       "y    -3.603181\n",
       "Name: 20, dtype: float64"
      ]
     },
     "execution_count": 15,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "df_data.iloc[20][[\"x\",\"y\"]]"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "The NumPy array `z` has shape `(2,)`.  Notice how this tuple has length 1, which corresponds to `z` being a one-dimensional NumPy array."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 16,
   "metadata": {
    "cell_id": "cb3cc4d8b69e42928082ec33f07c6f9c",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 12,
    "execution_start": 1667409246000,
    "source_hash": "266fcfea",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "(2,)"
      ]
     },
     "execution_count": 16,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "z.shape"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "It was asked in class what would happen if we take the transpose of `z`.  (Aside: I've never understood why we use the attribute `.T` to take a transpose in NumPy, as opposed to using the method version, `.T()`.)  In this case, where our NumPy array is one-dimensional, there is no change when we take the transpose."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 17,
   "metadata": {
    "cell_id": "54c5bbbe8f5640fab7fa16d836f6e628",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 365,
    "execution_start": 1667409316180,
    "source_hash": "5963d725",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "array([-10.4768696 ,  -3.60318139])"
      ]
     },
     "execution_count": 17,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "z.T"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 18,
   "metadata": {
    "cell_id": "1b249d80eb14490e9fe22642cf1b0dd0",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 5,
    "execution_start": 1667409323558,
    "source_hash": "860ac4bd",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "array([-10.4768696 ,  -3.60318139])"
      ]
     },
     "execution_count": 18,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "z"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 19,
   "metadata": {
    "cell_id": "3d9026ba03c9495282f6ce464d0aee62",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 662,
    "execution_start": 1667409362802,
    "source_hash": "c01b92d0",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "array([ True,  True])"
      ]
     },
     "execution_count": 19,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "z == z.T"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 20,
   "metadata": {
    "cell_id": "6a1b13c5e34c43fd833f48ce0855cded",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 4,
    "execution_start": 1667409382882,
    "source_hash": "1096ae71",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "True"
      ]
     },
     "execution_count": 20,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "(z == z.T).all()"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "cell_id": "1d0b06f28af24aabbb3afda8a059eeea",
    "deepnote_cell_type": "markdown",
    "tags": []
   },
   "source": [
    "## Clustering with scikit-learn\n",
    "\n",
    "Let's see how to make some of the same computations using scikit-learn.  With some more work above, we can get the exact same results by hand (but we didn't do that today)."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 21,
   "metadata": {
    "cell_id": "d2301e391a334c00a46025d2648b6f25",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 1,
    "execution_start": 1667409555553,
    "source_hash": "5c75adad",
    "tags": []
   },
   "outputs": [],
   "source": [
    "# import\n",
    "from sklearn.cluster import KMeans"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Here is a reminder of our starting points."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 22,
   "metadata": {
    "cell_id": "d4f6672e1ffa46699521028a80ce6e09",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 4,
    "execution_start": 1667409666204,
    "source_hash": "9baf55e4",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "array([[ -4, -10],\n",
       "       [ -4,  -8],\n",
       "       [-10, -10]])"
      ]
     },
     "execution_count": 22,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "centroids"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Because we want to use 3 starting points, we need to use `n_clusters=3`.  The `max_iter=1` means we only want scikit-learn to perform 1 step of the K-means clustering algorithm.  (It will still go further than we went above.)  The `n_init=1` is more confusing.  Usually scikit-learn will repeat the entire clustering algorithm multiple times, and then choose the best one.  We are telling it to only perform the entire clustering algorithm once (and in fact, that is necessary because if we are always using the same starting points `centroids`, then we will always get the same results)."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 23,
   "metadata": {
    "cell_id": "3582b9e207ea4b5da03d612cdb20cec2",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 0,
    "execution_start": 1667409938590,
    "source_hash": "d7846639",
    "tags": []
   },
   "outputs": [],
   "source": [
    "# instantiate\n",
    "kmeans = KMeans(n_clusters=3, init=centroids, max_iter=1, n_init=1)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 24,
   "metadata": {
    "cell_id": "4e0e3b66e2e04402aadf9070c7616566",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 15,
    "execution_start": 1667409940121,
    "source_hash": "2c6d42be",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>x</th>\n",
       "      <th>y</th>\n",
       "      <th>cluster</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>0</th>\n",
       "      <td>-6.923242</td>\n",
       "      <td>-10.669532</td>\n",
       "      <td>0</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>1</th>\n",
       "      <td>-8.630620</td>\n",
       "      <td>-7.139406</td>\n",
       "      <td>2</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2</th>\n",
       "      <td>-9.630481</td>\n",
       "      <td>-2.720449</td>\n",
       "      <td>2</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>3</th>\n",
       "      <td>-2.306477</td>\n",
       "      <td>5.307977</td>\n",
       "      <td>1</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>4</th>\n",
       "      <td>-7.570054</td>\n",
       "      <td>-3.014465</td>\n",
       "      <td>1</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "          x          y  cluster\n",
       "0 -6.923242 -10.669532        0\n",
       "1 -8.630620  -7.139406        2\n",
       "2 -9.630481  -2.720449        2\n",
       "3 -2.306477   5.307977        1\n",
       "4 -7.570054  -3.014465        1"
      ]
     },
     "execution_count": 24,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "df_data[:5]"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 25,
   "metadata": {
    "cell_id": "276ee4b5de584533a0756609aabd7c8a",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 134,
    "execution_start": 1667409941330,
    "source_hash": "ca38b02b",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>x</th>\n",
       "      <th>y</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>0</th>\n",
       "      <td>-6.923242</td>\n",
       "      <td>-10.669532</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>1</th>\n",
       "      <td>-8.630620</td>\n",
       "      <td>-7.139406</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2</th>\n",
       "      <td>-9.630481</td>\n",
       "      <td>-2.720449</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>3</th>\n",
       "      <td>-2.306477</td>\n",
       "      <td>5.307977</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>4</th>\n",
       "      <td>-7.570054</td>\n",
       "      <td>-3.014465</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>...</th>\n",
       "      <td>...</td>\n",
       "      <td>...</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>495</th>\n",
       "      <td>-7.827495</td>\n",
       "      <td>-2.510321</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>496</th>\n",
       "      <td>-6.380881</td>\n",
       "      <td>-8.506638</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>497</th>\n",
       "      <td>-8.960149</td>\n",
       "      <td>-8.063499</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>498</th>\n",
       "      <td>-7.666039</td>\n",
       "      <td>-7.597155</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>499</th>\n",
       "      <td>-6.465344</td>\n",
       "      <td>-2.855446</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "<p>500 rows × 2 columns</p>\n",
       "</div>"
      ],
      "text/plain": [
       "            x          y\n",
       "0   -6.923242 -10.669532\n",
       "1   -8.630620  -7.139406\n",
       "2   -9.630481  -2.720449\n",
       "3   -2.306477   5.307977\n",
       "4   -7.570054  -3.014465\n",
       "..        ...        ...\n",
       "495 -7.827495  -2.510321\n",
       "496 -6.380881  -8.506638\n",
       "497 -8.960149  -8.063499\n",
       "498 -7.666039  -7.597155\n",
       "499 -6.465344  -2.855446\n",
       "\n",
       "[500 rows x 2 columns]"
      ]
     },
     "execution_count": 25,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "df_data[[\"x\",\"y\"]]"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "We want to fit the `KMeans` object `kmeans` using just the \"x\" and \"y\" columns."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 26,
   "metadata": {
    "cell_id": "6b5a2fbe626b48818280ae8ab4a0a2a1",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 11,
    "execution_start": 1667409949177,
    "source_hash": "4cee84dd",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<style>#sk-container-id-1 {color: black;background-color: white;}#sk-container-id-1 pre{padding: 0;}#sk-container-id-1 div.sk-toggleable {background-color: white;}#sk-container-id-1 label.sk-toggleable__label {cursor: pointer;display: block;width: 100%;margin-bottom: 0;padding: 0.3em;box-sizing: border-box;text-align: center;}#sk-container-id-1 label.sk-toggleable__label-arrow:before {content: \"▸\";float: left;margin-right: 0.25em;color: #696969;}#sk-container-id-1 label.sk-toggleable__label-arrow:hover:before {color: black;}#sk-container-id-1 div.sk-estimator:hover label.sk-toggleable__label-arrow:before {color: black;}#sk-container-id-1 div.sk-toggleable__content {max-height: 0;max-width: 0;overflow: hidden;text-align: left;background-color: #f0f8ff;}#sk-container-id-1 div.sk-toggleable__content pre {margin: 0.2em;color: black;border-radius: 0.25em;background-color: #f0f8ff;}#sk-container-id-1 input.sk-toggleable__control:checked~div.sk-toggleable__content {max-height: 200px;max-width: 100%;overflow: auto;}#sk-container-id-1 input.sk-toggleable__control:checked~label.sk-toggleable__label-arrow:before {content: \"▾\";}#sk-container-id-1 div.sk-estimator input.sk-toggleable__control:checked~label.sk-toggleable__label {background-color: #d4ebff;}#sk-container-id-1 div.sk-label input.sk-toggleable__control:checked~label.sk-toggleable__label {background-color: #d4ebff;}#sk-container-id-1 input.sk-hidden--visually {border: 0;clip: rect(1px 1px 1px 1px);clip: rect(1px, 1px, 1px, 1px);height: 1px;margin: -1px;overflow: hidden;padding: 0;position: absolute;width: 1px;}#sk-container-id-1 div.sk-estimator {font-family: monospace;background-color: #f0f8ff;border: 1px dotted black;border-radius: 0.25em;box-sizing: border-box;margin-bottom: 0.5em;}#sk-container-id-1 div.sk-estimator:hover {background-color: #d4ebff;}#sk-container-id-1 div.sk-parallel-item::after {content: \"\";width: 100%;border-bottom: 1px solid gray;flex-grow: 1;}#sk-container-id-1 div.sk-label:hover label.sk-toggleable__label {background-color: #d4ebff;}#sk-container-id-1 div.sk-serial::before {content: \"\";position: absolute;border-left: 1px solid gray;box-sizing: border-box;top: 0;bottom: 0;left: 50%;z-index: 0;}#sk-container-id-1 div.sk-serial {display: flex;flex-direction: column;align-items: center;background-color: white;padding-right: 0.2em;padding-left: 0.2em;position: relative;}#sk-container-id-1 div.sk-item {position: relative;z-index: 1;}#sk-container-id-1 div.sk-parallel {display: flex;align-items: stretch;justify-content: center;background-color: white;position: relative;}#sk-container-id-1 div.sk-item::before, #sk-container-id-1 div.sk-parallel-item::before {content: \"\";position: absolute;border-left: 1px solid gray;box-sizing: border-box;top: 0;bottom: 0;left: 50%;z-index: -1;}#sk-container-id-1 div.sk-parallel-item {display: flex;flex-direction: column;z-index: 1;position: relative;background-color: white;}#sk-container-id-1 div.sk-parallel-item:first-child::after {align-self: flex-end;width: 50%;}#sk-container-id-1 div.sk-parallel-item:last-child::after {align-self: flex-start;width: 50%;}#sk-container-id-1 div.sk-parallel-item:only-child::after {width: 0;}#sk-container-id-1 div.sk-dashed-wrapped {border: 1px dashed gray;margin: 0 0.4em 0.5em 0.4em;box-sizing: border-box;padding-bottom: 0.4em;background-color: white;}#sk-container-id-1 div.sk-label label {font-family: monospace;font-weight: bold;display: inline-block;line-height: 1.2em;}#sk-container-id-1 div.sk-label-container {text-align: center;}#sk-container-id-1 div.sk-container {/* jupyter's `normalize.less` sets `[hidden] { display: none; }` but bootstrap.min.css set `[hidden] { display: none !important; }` so we also need the `!important` here to be able to override the default hidden behavior on the sphinx rendered scikit-learn.org. See: https://github.com/scikit-learn/scikit-learn/issues/21755 */display: inline-block !important;position: relative;}#sk-container-id-1 div.sk-text-repr-fallback {display: none;}</style><div id=\"sk-container-id-1\" class=\"sk-top-container\"><div class=\"sk-text-repr-fallback\"><pre>KMeans(init=array([[ -4, -10],\n",
       "       [ -4,  -8],\n",
       "       [-10, -10]]),\n",
       "       max_iter=1, n_clusters=3, n_init=1)</pre><b>In a Jupyter environment, please rerun this cell to show the HTML representation or trust the notebook. <br />On GitHub, the HTML representation is unable to render, please try loading this page with nbviewer.org.</b></div><div class=\"sk-container\" hidden><div class=\"sk-item\"><div class=\"sk-estimator sk-toggleable\"><input class=\"sk-toggleable__control sk-hidden--visually\" id=\"sk-estimator-id-1\" type=\"checkbox\" checked><label for=\"sk-estimator-id-1\" class=\"sk-toggleable__label sk-toggleable__label-arrow\">KMeans</label><div class=\"sk-toggleable__content\"><pre>KMeans(init=array([[ -4, -10],\n",
       "       [ -4,  -8],\n",
       "       [-10, -10]]),\n",
       "       max_iter=1, n_clusters=3, n_init=1)</pre></div></div></div></div></div>"
      ],
      "text/plain": [
       "KMeans(init=array([[ -4, -10],\n",
       "       [ -4,  -8],\n",
       "       [-10, -10]]),\n",
       "       max_iter=1, n_clusters=3, n_init=1)"
      ]
     },
     "execution_count": 26,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "# fit\n",
    "kmeans.fit(df_data[[\"x\",\"y\"]])"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Here are the predicted clusters."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 27,
   "metadata": {
    "cell_id": "c3cc526b86364925bbf8ba3cabdd6cb0",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 6,
    "execution_start": 1667410029270,
    "source_hash": "5b67354f",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "array([0, 2, 2, 1, 2, 1, 1, 1, 2, 1, 0, 0, 1, 1, 2, 2, 2, 0, 1, 1, 2, 2,\n",
       "       1, 1, 1, 2, 1, 1, 2, 1, 0, 1, 1, 0, 1, 0, 2, 2, 0, 2, 1, 2, 1, 1,\n",
       "       1, 0, 2, 1, 1, 1, 1, 2, 2, 1, 0, 2, 2, 2, 2, 1, 1, 1, 2, 2, 1, 1,\n",
       "       0, 1, 2, 2, 0, 2, 1, 0, 1, 1, 0, 1, 1, 2, 1, 1, 2, 1, 1, 2, 2, 1,\n",
       "       2, 2, 1, 2, 2, 2, 1, 0, 0, 1, 0, 1, 1, 0, 1, 2, 1, 2, 1, 1, 1, 2,\n",
       "       1, 0, 1, 2, 1, 2, 1, 2, 1, 1, 2, 2, 0, 2, 2, 2, 1, 0, 0, 1, 1, 2,\n",
       "       2, 2, 2, 2, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 0, 2, 2, 2, 1, 0,\n",
       "       1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 2, 0, 0, 1, 1, 0, 2, 1, 2, 1, 2,\n",
       "       2, 2, 2, 0, 1, 2, 1, 0, 0, 2, 1, 0, 0, 2, 1, 1, 0, 2, 1, 1, 1, 0,\n",
       "       0, 1, 0, 2, 1, 1, 1, 0, 1, 2, 0, 1, 2, 2, 2, 0, 1, 1, 1, 1, 2, 1,\n",
       "       1, 0, 2, 1, 2, 2, 2, 1, 2, 1, 0, 2, 0, 2, 1, 1, 2, 0, 1, 0, 1, 2,\n",
       "       0, 0, 0, 0, 1, 2, 0, 2, 1, 1, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2,\n",
       "       1, 1, 0, 0, 2, 2, 2, 1, 1, 1, 1, 1, 2, 2, 1, 0, 1, 0, 0, 1, 0, 1,\n",
       "       1, 2, 2, 2, 2, 1, 1, 2, 0, 1, 0, 2, 2, 1, 2, 1, 0, 2, 0, 2, 2, 0,\n",
       "       0, 2, 1, 0, 1, 1, 1, 1, 2, 0, 1, 1, 1, 0, 1, 1, 2, 2, 1, 2, 1, 0,\n",
       "       0, 1, 2, 0, 2, 1, 1, 2, 2, 1, 0, 1, 1, 1, 2, 1, 1, 1, 1, 0, 0, 1,\n",
       "       1, 2, 1, 0, 1, 2, 1, 1, 1, 2, 2, 2, 1, 2, 1, 0, 1, 0, 0, 0, 0, 0,\n",
       "       0, 2, 0, 1, 0, 1, 1, 2, 1, 1, 2, 2, 0, 1, 2, 1, 1, 0, 2, 2, 0, 2,\n",
       "       2, 1, 1, 0, 2, 1, 1, 1, 1, 0, 0, 1, 0, 2, 2, 0, 2, 0, 2, 1, 0, 1,\n",
       "       2, 1, 1, 2, 1, 1, 0, 1, 1, 1, 2, 1, 2, 1, 0, 1, 2, 1, 1, 0, 0, 0,\n",
       "       2, 1, 2, 0, 1, 1, 2, 2, 0, 1, 1, 2, 1, 1, 1, 2, 1, 2, 0, 1, 1, 2,\n",
       "       1, 1, 2, 2, 1, 1, 1, 2, 1, 2, 0, 2, 1, 1, 0, 1, 2, 1, 0, 1, 1, 1,\n",
       "       0, 2, 2, 2, 2, 2, 2, 0, 1, 1, 2, 2, 0, 2, 0, 1], dtype=int32)"
      ]
     },
     "execution_count": 27,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "# predict\n",
    "kmeans.predict(df_data[[\"x\", \"y\"]])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 28,
   "metadata": {
    "cell_id": "79af55d2c8ee4347b3ca04916e12b0cf",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 3,
    "execution_start": 1667410078918,
    "source_hash": "3fe668ea",
    "tags": []
   },
   "outputs": [],
   "source": [
    "df_data[\"cluster-sklearn\"] = kmeans.predict(df_data[[\"x\", \"y\"]])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 29,
   "metadata": {
    "cell_id": "14b31bb3c5ba4f22b5d2856f8de6b7f9",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 4,
    "execution_start": 1667410083736,
    "source_hash": "2c6d42be",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>x</th>\n",
       "      <th>y</th>\n",
       "      <th>cluster</th>\n",
       "      <th>cluster-sklearn</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>0</th>\n",
       "      <td>-6.923242</td>\n",
       "      <td>-10.669532</td>\n",
       "      <td>0</td>\n",
       "      <td>0</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>1</th>\n",
       "      <td>-8.630620</td>\n",
       "      <td>-7.139406</td>\n",
       "      <td>2</td>\n",
       "      <td>2</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2</th>\n",
       "      <td>-9.630481</td>\n",
       "      <td>-2.720449</td>\n",
       "      <td>2</td>\n",
       "      <td>2</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>3</th>\n",
       "      <td>-2.306477</td>\n",
       "      <td>5.307977</td>\n",
       "      <td>1</td>\n",
       "      <td>1</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>4</th>\n",
       "      <td>-7.570054</td>\n",
       "      <td>-3.014465</td>\n",
       "      <td>1</td>\n",
       "      <td>2</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "</div>"
      ],
      "text/plain": [
       "          x          y  cluster  cluster-sklearn\n",
       "0 -6.923242 -10.669532        0                0\n",
       "1 -8.630620  -7.139406        2                2\n",
       "2 -9.630481  -2.720449        2                2\n",
       "3 -2.306477   5.307977        1                1\n",
       "4 -7.570054  -3.014465        1                2"
      ]
     },
     "execution_count": 29,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "df_data[:5]"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Let's now make the same chart as above, but this time coloring using the scikit-learn clusters.  The chart `c2` still shows the original starting points."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 30,
   "metadata": {
    "cell_id": "a55000fc89c44fad993ebf1c9fb1c05a",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 877,
    "execution_start": 1667410144241,
    "source_hash": "5b09d3fc",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<div id=\"altair-viz-74fcb22ffabe4c9798c1ef7539317010\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-74fcb22ffabe4c9798c1ef7539317010\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-74fcb22ffabe4c9798c1ef7539317010\");\n",
       "    }\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm//vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm//vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm//vega-lite@4.17.0?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm//vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      require([\"vega-embed\"], displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"4.17.0\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 400, \"continuousHeight\": 300}}, \"layer\": [{\"data\": {\"name\": \"data-12bb0cfa88e3591c947140beaa35a2c9\"}, \"mark\": \"circle\", \"encoding\": {\"color\": {\"field\": \"cluster-sklearn\", \"type\": \"nominal\"}, \"x\": {\"field\": \"x\", \"type\": \"quantitative\"}, \"y\": {\"field\": \"y\", \"type\": \"quantitative\"}}}, {\"data\": {\"name\": \"data-d937417af026e4f584a32531c2ef6637\"}, \"mark\": {\"type\": \"point\", \"filled\": true, \"shape\": \"cross\", \"size\": 500, \"stroke\": \"black\"}, \"encoding\": {\"x\": {\"field\": \"x\", \"type\": \"quantitative\"}, \"y\": {\"field\": \"y\", \"type\": \"quantitative\"}}}], \"$schema\": \"https://vega.github.io/schema/vega-lite/v4.17.0.json\", \"datasets\": {\"data-12bb0cfa88e3591c947140beaa35a2c9\": [{\"x\": -6.923241645991934, \"y\": -10.669531965589794, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -8.630620334313644, \"y\": -7.13940563765234, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.630480689769236, \"y\": -2.7204493549869673, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.3064765942034278, \"y\": 5.307976760491872, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.570053657727669, \"y\": -3.0144649054564128, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.0005101098483093, \"y\": 2.779051528226904, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -4.818268385973806, \"y\": -2.772148223435087, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.339647992855507, \"y\": -1.276257644021504, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.943088401134986, \"y\": -3.899939010564857, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -5.549245254367849, \"y\": -3.412989678624256, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.145089897229651, \"y\": -9.544921980678078, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -7.09669935635653, \"y\": -8.04074036220985, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.826415121664895, \"y\": -1.963461960474791, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.8319881134989553, \"y\": 3.5286314509217895, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.342672348933138, \"y\": -3.1654648218447767, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -7.340728246218882, \"y\": -6.924272518973348, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -7.946539060795657, \"y\": -3.3676865471908224, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -8.245985356429562, \"y\": -8.613158209058627, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -1.9819771099620271, \"y\": 4.022435514174746, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -4.350980346475657, \"y\": -3.6947667796776424, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.476869602439182, \"y\": -3.6031813885349258, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -11.01959844829441, \"y\": -3.1588203077621224, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.17255903986124, \"y\": -4.318359712685473, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.406718199699357, \"y\": 6.098944469870908, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.721494975923703, \"y\": -2.8844080642178818, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.58935963436387, \"y\": -4.43379547940742, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.4612601858217498, \"y\": 4.525498514650621, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.9190034548522448, \"y\": 3.452789267035227, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.409351666841584, \"y\": -2.674820464521483, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.367228090988169, \"y\": -3.326660724595877, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.72766124534928, \"y\": -7.145162671548282, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -2.279560749896649, \"y\": 5.104521902915381, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.848875595943767, \"y\": -3.0397050619974144, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.079930511938528, \"y\": -7.081975676778154, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.266829294846004, \"y\": -2.696450548014737, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.05367511988283, \"y\": -9.629790765144225, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -10.08222053105773, \"y\": -4.250710430136708, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -11.870873481288402, \"y\": -3.0327334293082484, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.371073070369658, \"y\": -7.9563583253737535, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -9.375908999347528, \"y\": -4.553153078434052, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.6340198681573215, \"y\": -2.583403556386127, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.54609655309786, \"y\": -2.8491742210599913, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.6982554152145706, \"y\": 2.7907175141398666, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.602176024325104, \"y\": -0.6599084896160656, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.03429022029922, \"y\": -4.088211960497224, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.372307836118383, \"y\": -8.631900461039706, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.226478332523264, \"y\": -2.339987172800089, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.956781476027413, \"y\": -3.979057011247294, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.4270653467697816, \"y\": 5.089041275707647, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.6207353040891055, \"y\": 6.59346951581742, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.32810279336092774, \"y\": 4.119182005367143, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.62305450511909, \"y\": -4.547191614617715, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.12674270112086, \"y\": -4.461805681638605, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.241344969182345, \"y\": -3.2350587337503987, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.199675312953462, \"y\": -7.104009813871477, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.113697689125498, \"y\": -4.128807516024957, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.341613212863495, \"y\": -4.95351773723572, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -12.504153156329163, \"y\": -6.067512469280031, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.323316398832233, \"y\": -4.675740452777355, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.320330019823476, \"y\": -2.733500952132815, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.205334073931087, \"y\": 4.207652005954265, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.5279305184970926, \"y\": 5.92630668526536, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.874300341631166, \"y\": -3.648081507119194, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.66948545344214, \"y\": -4.260598838360167, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.3739725806942609, \"y\": 5.291631033113889, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.600857078206719, \"y\": -3.1196968767431805, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.991754122866363, \"y\": -8.335648510987143, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.224478686299447, \"y\": -2.438462244886699, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -11.105425005937654, \"y\": -3.9710668652733467, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.957623350499006, \"y\": -4.87178858566323, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.654616438260894, \"y\": -7.293357132827133, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.95313776602679, \"y\": -3.367438116260589, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.113510079104775, \"y\": -2.0188199201889114, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.24251437964129, \"y\": -9.663684479711614, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.349294557744197, \"y\": -3.545773316732582, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.2926133215005216, \"y\": -3.6889242583746578, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.353879526667555, \"y\": -8.545044344432092, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.784234728636943, \"y\": -4.484068484204371, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.662563249318529, \"y\": -2.343900922698393, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.608933107282834, \"y\": -4.614692790516223, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -0.25201990570729094, \"y\": 4.535591448553887, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.9274479855745354, \"y\": 4.9368453355813475, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.413065893175675, \"y\": -3.6290743039147837, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -0.8486082331449638, \"y\": 5.4509319635504205, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.8667530400291719, \"y\": 3.7829591391633697, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.184291497164567, \"y\": -4.010173028249052, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.931929183434267, \"y\": -5.424505473195829, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -2.7544717516895614, \"y\": 4.575872301901829, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -11.717106978272625, \"y\": -3.8962275512811124, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.850812127253619, \"y\": -4.00305112652188, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.3439249587069142, \"y\": 2.3842886530191585, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.1620365439131, \"y\": -7.314593357395667, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.188868143387419, \"y\": -2.163593864156933, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -7.132292597186354, \"y\": -4.022967302286994, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -0.04261030709121938, \"y\": 4.909230750843066, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.244494478309097, \"y\": -7.65150299527888, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -8.137846462369383, \"y\": -7.658069493423728, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.7345134534846505, \"y\": -1.3833019413836023, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.963694242977034, \"y\": -9.270338797063157, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": 0.8687658008578785, \"y\": 4.1578550911277015, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.5451769291687276, \"y\": 3.819965930021569, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.016944284363998, \"y\": -8.6713736570228, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -3.3337557135674514, \"y\": 5.231519693282148, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -11.43858852794494, \"y\": -2.7210954765347686, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.5208762667245614, \"y\": 5.081201394531134, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.8439444259585525, \"y\": -4.150582220449893, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -5.876197384285145, \"y\": -3.2807891557700892, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.2181954623799376, \"y\": 4.3063346355596615, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.0034135815195953, \"y\": 4.450086725677409, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.107703993816708, \"y\": -3.94479960192718, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.030458537548547, \"y\": -1.237347564565535, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.9568513663453935, \"y\": -8.12381048565328, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -2.3302221922375503, \"y\": 4.784053655164081, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.984359827305925, \"y\": -4.648042142650431, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.3308060367853387, \"y\": 4.39382526992426, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.779624186339902, \"y\": -4.390857530191786, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.034844856038634, \"y\": 3.7677594613889394, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.167442445959466, \"y\": -3.249983778680282, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -0.49922133595474993, \"y\": 4.775982585218885, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.766811435831203, \"y\": -3.4128177881590513, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.699056929910277, \"y\": -4.490571571238543, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.2872962121876474, \"y\": -6.683067760798305, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -8.178318286630729, \"y\": -8.220638129570899, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -9.144431281697479, \"y\": -4.366377857074236, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.223235428900379, \"y\": -3.5122637563645807, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -9.712964392804043, \"y\": -3.690881101899936, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -0.03190915279961315, \"y\": 4.744501565417638, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.104060439867236, \"y\": -8.381982275844257, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.5248250144984254, \"y\": -7.508874438593723, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.31161342896361, \"y\": -2.9764169703468255, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.5381421977128555, \"y\": 4.815390406779989, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.580410498678239, \"y\": -3.168577896347615, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.53106923642426, \"y\": -2.9196616794237036, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.765022329984836, \"y\": -3.278777840407201, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.546589564918502, \"y\": -4.648269448548444, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.3939337329988195, \"y\": -6.806122639953547, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -2.9915115724657677, \"y\": 2.645801308444378, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.675582536987324, \"y\": -4.559022548160358, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -3.5175417703952716, \"y\": 5.642653899295982, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.9985396183498266, \"y\": 6.198648077818729, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.964979013839094, \"y\": -2.0374646912902934, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.852795074857454, \"y\": -7.791380788784477, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -4.643104258815785, \"y\": -2.227894218120615, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.359389585992692, \"y\": 4.054240022349643, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.5257904636130821, \"y\": 3.3065986015291307, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.1563750925131935, \"y\": 5.6997157513754235, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.425300098097034, \"y\": -2.173286191076472, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.701833046489826, \"y\": -2.630838382829303, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.046329708889857, \"y\": -6.92266990466932, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -8.145592876103246, \"y\": -7.42775410014251, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.156850945511998, \"y\": -4.056235764936742, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.161707784218832, \"y\": -2.409989436558721, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.4686444212810534, \"y\": 6.506745005322004, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.746727983979049, \"y\": -8.172459738721683, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.9860594040267037, \"y\": 3.0638140794657254, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.32899567358573, \"y\": -3.5668094022301133, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.343132351066401, \"y\": -4.004536994970869, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.559546156361863, \"y\": -2.831020225706112, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.165911343397818, \"y\": -4.127528891671195, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.841448653383019, \"y\": -4.143569572448066, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.276810225196392, \"y\": -2.330499461409204, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.103057216160643, \"y\": -3.323152876678294, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.902287416461844, \"y\": -3.031898478703693, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.721213198830919, \"y\": -4.686620151258517, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.8513954583101344, \"y\": 3.5188609047583252, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.6932118924458734, \"y\": -6.300218619554666, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.533718390201513, \"y\": -8.149227262648669, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -8.463695002324215, \"y\": -8.071460285818429, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.750045280149759, \"y\": -3.565909665608377, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.1710417594110225, \"y\": 4.330918155822106, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.526285788379845, \"y\": -8.669576013270067, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -9.23890684355517, \"y\": -3.0684397328983213, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.128030509059693, \"y\": -2.516980580274055, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.10406450569264, \"y\": -7.420204872904966, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.6158909127701304, \"y\": 4.180175626329556, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.987585326318378, \"y\": -3.0333306141154415, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -11.941035909321455, \"y\": -3.6008541829938716, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.439941783166274, \"y\": -3.6298211937617726, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -11.424267924173634, \"y\": -2.185388599052178, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.009929142269778, \"y\": -9.068652472855751, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.474356490442799, \"y\": -3.7433886276640225, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.631380489113043, \"y\": -4.9979379292709964, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": 0.5260155005846419, \"y\": 3.009993533355024, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.763243925323785, \"y\": -9.366566230452749, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.2796552640230505, \"y\": -8.818095871591257, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -9.468832758067927, \"y\": -6.190435058475675, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.773366182553252, \"y\": -3.5673995330002293, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.692425332146296, \"y\": -8.301717907683438, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.444399699191618, \"y\": -9.168031795482715, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -7.11478468705883, \"y\": -5.3869913396768965, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -0.3858039757897429, \"y\": 6.373591618097896, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.004547116103499, \"y\": 4.175650125707692, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.755176277975085, \"y\": -9.30821073955341, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -9.141684205581292, \"y\": -7.205726937278369, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.920925345563742, \"y\": -3.2757404756717943, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.351220657673829, \"y\": 4.0097363419871845, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.919078510870565, \"y\": -2.2391986140442226, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.6220052551614295, \"y\": -8.692909666826537, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.54246303831446, \"y\": -8.127228110438386, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -2.4139578469451726, \"y\": 5.659358024076449, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.371515957997354, \"y\": -8.911295433180277, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -12.14017923273047, \"y\": -4.783517411385913, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -4.45264490609506, \"y\": 6.344018682459241, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.596988201326771, \"y\": -4.19535852653093, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.075036217253955, \"y\": -2.156064045450953, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.248282380977763, \"y\": -7.052227901955563, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -4.778911006562106, \"y\": -2.4133316499348325, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -12.411215504826888, \"y\": -5.730914923169192, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.7526434946447385, \"y\": -8.346549745098049, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.054921393415961, \"y\": -4.222577494532622, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.382544752485892, \"y\": -2.4952403087536847, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.225705023497352, \"y\": -3.7931357906625034, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -11.949817790898333, \"y\": -5.355677692816341, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.628670918240042, \"y\": -8.063541697301455, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -4.617671134403657, \"y\": -1.6711114461396843, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.122196637435978, \"y\": -3.3130212312625433, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.29225072428276, \"y\": -2.357382942013979, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": 0.00024227116135100424, \"y\": 5.148534029420497, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.79988166193185, \"y\": -2.2487543759907975, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -2.7768702545837973, \"y\": 4.640905566660254, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.396949787558266, \"y\": -3.769637033156362, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.9226308111314205, \"y\": -7.639722624635498, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -11.576868789047566, \"y\": -4.78197652621961, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.668247372033288, \"y\": -3.8260750868989764, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -11.157882608635378, \"y\": -2.6032417286814784, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.473085378791083, \"y\": -3.4757383655456473, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.981184936347486, \"y\": -3.776160827087244, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.0410207751622265, \"y\": 3.9633179381425236, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.328560152872887, \"y\": -2.6089330879852275, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.1389835688751488, \"y\": 3.262148479220019, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.179056384691831, \"y\": -7.9633664552409495, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.235654439405284, \"y\": -2.798060661274979, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.771332562994431, \"y\": -8.592225768290323, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -9.145008441927814, \"y\": -3.9179884503084472, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.6173461592329268, \"y\": 4.9893050825589835, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.7786753045082477, \"y\": 6.362568772546803, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.54642848761293, \"y\": -5.637408533231442, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.914865904929935, \"y\": -7.6896937828923795, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.8461296778558682, \"y\": 4.304743996314016, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.5283458556366405, \"y\": -8.1536031133824, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -6.009153369932298, \"y\": -3.3492515180044244, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.546283243062167, \"y\": -4.571385395044784, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.316556391289163, \"y\": -7.770512930899681, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.20423398910866, \"y\": -8.881765594048746, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -7.556007321448557, \"y\": -8.018854988584291, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.67856791941142, \"y\": -7.605098519935852, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.214468263289775, \"y\": -4.799953118763537, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.376629802814094, \"y\": -2.9972268432385083, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.318447094429766, \"y\": -8.92829839169748, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.827884398968216, \"y\": -4.833926147207375, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.065699095871062, \"y\": -1.5337694648059559, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.346732606068119, \"y\": 3.561284227344442, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.2560682641577197, \"y\": 5.0000683920802285, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.839797448183742, \"y\": -2.178361856135961, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.8708821070342445, \"y\": -2.227162357797841, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.7960046498445525, \"y\": 4.287435681068351, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.379726969732767, \"y\": -4.13752487301695, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.236059371477093, \"y\": -4.547109919532199, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -10.279448772718714, \"y\": -1.8969930241649111, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.4168904648667249, \"y\": 4.608320047717165, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.780454121464204, \"y\": -4.582979224413729, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": 0.08525185826796045, \"y\": 3.6452829679480585, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.202686409228257, \"y\": -4.327786866206425, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.568186363631137, \"y\": -4.560346945563251, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.1643485819705592, \"y\": 4.2317867091006605, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.163458506063613, \"y\": -3.108308017459318, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.321525640143877, \"y\": -9.662800788848088, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -7.520999738806424, \"y\": -9.133118356285845, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -9.220293298404238, \"y\": -4.0721197194662935, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.849168160183929, \"y\": -2.9524671236829034, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.863664307098478, \"y\": -2.751293685163499, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.797152237582278, \"y\": -3.458041360968013, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.979490066070415, \"y\": 4.08668826994449, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.060438097737446, \"y\": 5.230495487293569, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.668391828142745, \"y\": -0.7950678467818668, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.757969185355724, \"y\": 4.908984207745029, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.420569465043878, \"y\": -3.866884140838917, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.124250086206238, \"y\": -6.704238700465809, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.3788948288258804, \"y\": 4.333377165095884, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.614664441619893, \"y\": -7.525791021870934, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.340520809891421, \"y\": 4.157119493365752, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.211599997834005, \"y\": -8.292939835065118, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.5688561328000565, \"y\": -8.135272212973218, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -1.7700069323658358, \"y\": 3.789127805130801, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.365858337613077, \"y\": -7.345772187618775, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.499522836500689, \"y\": 5.282658789959386, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.8588279382837056, \"y\": 5.2698351863871835, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.738849346027174, \"y\": -3.2432766529495733, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -10.820179692557353, \"y\": -3.231637255250512, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.536820123190024, \"y\": -3.3608757459391847, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -12.034913729333896, \"y\": -5.895937728839661, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.269109092135366, \"y\": -2.7352182431364156, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.712996039785251, \"y\": -2.9032498381724894, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.361186337903108, \"y\": -2.726983818567998, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -5.489414275085358, \"y\": -6.946620206588477, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": 0.5311398230212139, \"y\": 2.510128945952069, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.6412677543087515, \"y\": -7.249228930620518, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -9.482638890013735, \"y\": -6.735883015016127, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.531037040483667, \"y\": -6.768236760975406, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.310785954173774, \"y\": -2.051746477365137, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.70233177666608, \"y\": -4.194625400586082, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.110130706597376, \"y\": -2.3106112800394785, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.839726334384508, \"y\": -9.206774178324359, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -11.753638143885953, \"y\": -3.2385589498773895, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.291994815507753, \"y\": -9.852561709346341, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -7.855682137432907, \"y\": -6.9295058926639355, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.196710687456825, \"y\": -2.0868771722110795, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.963565376976385, \"y\": -7.833571162679451, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.776804016652588, \"y\": -6.655119921047011, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.87499402257129, \"y\": -4.82113577023613, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.84048020802667, \"y\": 3.802569241049904, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.980674032726418, \"y\": -8.56048014764401, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.32066245851635, \"y\": -3.3075189181067537, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.179799662377421, \"y\": -3.0080344705160247, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.1766543638687477, \"y\": 3.4094630411928986, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.6732247181272776, \"y\": 4.620023770178603, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.938921711708721, \"y\": -3.5152140767725673, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.489374974228666, \"y\": -8.884759086354379, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -2.8964132824899167, \"y\": 5.282328796492461, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.8133992583634861, \"y\": 3.546973928011175, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.7775266738006055, \"y\": -2.851452759799448, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.248838499333678, \"y\": -8.765635077752869, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -3.103673711378109, \"y\": 3.90202400589671, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.0572406256528906, \"y\": 4.826772072490221, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.732150477880266, \"y\": -5.046954544738055, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -9.936962307853594, \"y\": -3.742223792355488, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -3.0326772261952755, \"y\": 4.721649260885453, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.703553011055385, \"y\": -2.760662479608511, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.684756308285753, \"y\": -3.768169242136279, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.62182374152277, \"y\": -8.76567023290839, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.671772938803755, \"y\": -9.977147957249239, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -1.9257784119521415, \"y\": 4.43910441553652, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.162994877003415, \"y\": -3.388965691868891, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -3.743803427487374, \"y\": -8.75345343500136, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.666012113538473, \"y\": -4.970196326305104, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -2.8896180416154813, \"y\": 4.957027364819377, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.3599584108697744, \"y\": 4.203095419878603, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.8049155726466335, \"y\": -3.4960254831509756, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -7.104806758014256, \"y\": -4.108305308544761, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.9668553912810705, \"y\": -3.128763922329747, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.313544948497254, \"y\": -8.012832668176694, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -4.471206790960085, \"y\": -3.5413104324230704, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.5394009534668904, \"y\": 5.023692978550581, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.6087521511724905, \"y\": 3.769494222273808, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.192769844529003, \"y\": -3.147955123131023, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.8020781039706595, \"y\": 4.057147146430284, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": 0.24509880239246162, \"y\": 5.517546567403667, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -3.31028117177262, \"y\": 3.515934284680113, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.841878032458153, \"y\": 3.740735350644035, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.75867611852229, \"y\": -8.75783107378842, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.995910561456476, \"y\": -8.112856673601206, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -4.9836068695957065, \"y\": -3.2052296074107085, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.8684541393232976, \"y\": 4.993113060025359, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.715036791404682, \"y\": -4.779445979210525, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.473733222730479, \"y\": -2.7868254137329, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.992630284501854, \"y\": -7.143440769676486, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.5377386349570825, \"y\": 5.535973776755081, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.446450536324724, \"y\": -4.625796586149801, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.0967988093294934, \"y\": 4.647226961080894, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.252568773979549, \"y\": -2.9168283269714537, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.19745196890354544, \"y\": 2.3463491593455075, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.067041200343585, \"y\": -4.0617406145367445, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.13468589114183, \"y\": -4.507934236283532, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -10.372517190955755, \"y\": -4.703318164638871, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.8818880485588787, \"y\": 4.205731799913164, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.154984836988162, \"y\": -3.1077859806863612, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.142547993526798, \"y\": -3.652022063256501, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.427494271234997, \"y\": -9.6383845559495, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -1.1300945815958103, \"y\": 4.544191078840548, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.284855048319109, \"y\": -8.782669708677824, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.3332534877783155, \"y\": -8.284903730911063, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.403201107191756, \"y\": -7.166875924411981, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.2218758561510565, \"y\": -9.488430827639236, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.098342933428682, \"y\": -7.440179054320776, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.208077928664976, \"y\": -7.120244325674294, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -9.687440215869334, \"y\": -6.047596363586017, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.873729380129867, \"y\": -7.595788651766012, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -1.1466300855305107, \"y\": 4.108397033740446, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.903442201983345, \"y\": -8.180757490892024, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -2.760179083161441, \"y\": 5.551213578682775, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.2360655518844066, \"y\": 4.483829937191721, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.97584966702655, \"y\": -4.422022363641096, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.1066884707345177, \"y\": 5.63099757364866, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -4.73558876279152, \"y\": -4.237489688775991, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.723309622787529, \"y\": -4.821117224986795, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.26074369398222, \"y\": -5.647247819305372, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.883843439299868, \"y\": -7.046052653674982, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -2.1577734695795976, \"y\": 4.095504890540311, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.859884436213619, \"y\": -4.73888254428344, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -0.46064202604701254, \"y\": 4.591646286327105, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.0568548716452355, \"y\": -5.029466423503938, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.660550064568029, \"y\": -8.462349423265907, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -8.419239819175033, \"y\": -3.458347884476478, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -10.994732299182784, \"y\": -4.06014253414243, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.7137652871467735, \"y\": -8.221998567702684, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.797259994344861, \"y\": -4.244943142864037, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.237463277204007, \"y\": -4.014001039138014, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -2.932118661194514, \"y\": 4.720037589306593, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.6614513887865232, \"y\": 3.0098694442477294, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.657343468811008, \"y\": -10.458136042394184, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -9.980547777279012, \"y\": -4.382490825598298, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.519403743255727, \"y\": -2.3878033376158005, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.9696766794665796, \"y\": 1.9716521047360227, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -3.884649810113071, \"y\": -2.843362609949925, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.82969906210201, \"y\": -2.9906732129170535, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.667001758775887, \"y\": -9.149238993712444, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -6.628895987361329, \"y\": -8.840715497684219, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -6.489449614672605, \"y\": -2.0675373330401436, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.171342305213279, \"y\": -10.944224511317891, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -11.30424657649712, \"y\": -3.8769680672036126, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.536548403676084, \"y\": -5.12933121880452, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.098661321079398, \"y\": -7.427311252103224, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -8.789256175101139, \"y\": -2.837646744578718, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -7.323865037058271, \"y\": -7.963934907065387, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.033080428531813, \"y\": -1.842743493762495, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.0361977293704867, \"y\": 3.971533185887934, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.428298774243108, \"y\": -6.743974715260037, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -2.8793043029275522, \"y\": 6.855858517904559, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.52994652455128, \"y\": -2.835215150645509, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.114230780704071, \"y\": -3.2089354347677936, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.7824501314671677, \"y\": 3.4707204345840927, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.952718093979204, \"y\": -3.344833848878355, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.166179014456907, \"y\": -3.7917058620492132, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.6421504976485148, \"y\": 3.284471140096273, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.335342960043219, \"y\": -7.870232570813044, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.311077061000285, \"y\": -3.921180806827734, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.7800244777370269, \"y\": 3.1733691333967347, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.6841768618242978, \"y\": 3.631328249674002, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.555207225439144, \"y\": -3.0141798031667166, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.343540093678463, \"y\": -2.138976643927715, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -11.536505749397744, \"y\": -4.401243732079323, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -4.895037580840365, \"y\": -2.486334557899435, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.443969898035991, \"y\": -8.959412922006269, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.5817387780358136, \"y\": 5.024870130841686, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.0299385942939985, \"y\": -6.699310523015227, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.170742382737493, \"y\": -2.5607820370672085, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.2218653369676096, \"y\": 6.361367943852196, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.5738544614806, \"y\": -8.319714059510003, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -7.658225937900157, \"y\": -7.642920507878789, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.895012928310167, \"y\": -9.317236075785248, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -11.114182521537845, \"y\": -3.872421446389926, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -0.794152276623841, \"y\": 2.104951171962879, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.428031934213674, \"y\": -5.521293971786157, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -5.897807017657774, \"y\": -8.192896804714527, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -6.5916969689749285, \"y\": -2.4477995927555383, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.457857759408556, \"y\": -3.3098143609273802, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.775571330280513, \"y\": -2.837507439439036, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.234149455466088, \"y\": -3.2255350473635556, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.266818388358796, \"y\": -8.255160143887645, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.205809800827664, \"y\": -3.2985383945615987, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.460452643297657, \"y\": -2.3083155293806703, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.0425995184711425, \"y\": -3.453323508845682, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.099628043228513, \"y\": -3.142269151874201, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.660069496619134, \"y\": -3.437769650054597, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.080973984400039, \"y\": -3.039723766765942, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.412647124911402, \"y\": -6.682488247932676, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.365134104901397, \"y\": -1.3885973124935187, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.416650401331225, \"y\": -4.4325334624906105, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.416238539567072, \"y\": -8.045884810800455, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.889193480102257, \"y\": -2.370494723384702, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.4294651705841361, \"y\": 5.168501049155187, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.561180687841756, \"y\": -3.959673108791006, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.4729985149602352, \"y\": 4.816541516051418, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.881008041197264, \"y\": -3.316926149839501, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.412559404941337, \"y\": -3.5014025100419817, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.552093769106948, \"y\": -3.158409998594174, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -0.7906737489426516, \"y\": 5.156901508708169, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.10075436532866022, \"y\": 4.515892565268589, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.3090139272844121, \"y\": 3.09420645760573, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.547556990885923, \"y\": -2.188013446844133, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.320300105898895, \"y\": -2.99303868802891, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.48229870186986, \"y\": -5.068219599729142, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.743616269173907, \"y\": -8.87844303036941, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.25189237126047, \"y\": -2.5535046048822188, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.9657639185768914, \"y\": 5.234464511450407, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.88036774243447, \"y\": -2.363262900885486, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.347745735292583, \"y\": -8.419554988942766, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -0.7587039566841077, \"y\": 3.7227620096688283, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.413578630454982, \"y\": -6.850692568137864, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -0.8205764920740146, \"y\": 5.337591950146718, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.934890406848591, \"y\": -7.784037635077171, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.694465659843353, \"y\": -4.062053044388285, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.8576988740771748, \"y\": 4.453057167267307, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": 0.15097500817693676, \"y\": 3.100762946285424, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.553944409165649, \"y\": -6.44256626595291, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.931627159279603, \"y\": -4.4863688734468035, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.501555964616166, \"y\": -4.653293312950241, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.9365051935017945, \"y\": -6.392812921048723, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -10.13368982975369, \"y\": -4.750618332856334, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.891489779617883, \"y\": -5.479028863592562, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.898716170905923, \"y\": -4.854983037839203, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.113949928826354, \"y\": -7.836569209467391, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.290783535506342, \"y\": -3.6484668810576015, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.410760744560749, \"y\": 4.109848716472299, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.505375947716452, \"y\": -4.63402668837209, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.827494555623418, \"y\": -2.5103210388828026, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.380880855475786, \"y\": -8.506638087491059, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -8.960149131134205, \"y\": -8.063498987179393, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.666038984307122, \"y\": -7.59715459454666, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.465344070108034, \"y\": -2.855446332878923, \"cluster\": 1, \"cluster-sklearn\": 1}], \"data-d937417af026e4f584a32531c2ef6637\": [{\"x\": -4, \"y\": -10}, {\"x\": -4, \"y\": -8}, {\"x\": -10, \"y\": -10}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.LayerChart(...)"
      ]
     },
     "execution_count": 30,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "c1 = alt.Chart(df_data).mark_circle().encode(\n",
    "    x=\"x\",\n",
    "    y=\"y\",\n",
    "    color=\"cluster-sklearn:N\"\n",
    ")\n",
    "\n",
    "c2 = alt.Chart(df_centroids).mark_point(\n",
    "    size=500,\n",
    "    shape='cross',\n",
    "    filled=True,\n",
    "    stroke=\"black\",\n",
    ").encode(\n",
    "    x=\"x\",\n",
    "    y=\"y\",\n",
    ")\n",
    "\n",
    "c1+c2"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Let's update the chart to show the next centers.  (Remember, each step of the K-means clustering algorithm, new centers are computed.)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 31,
   "metadata": {
    "cell_id": "8c0877c8871d440db69d44e917278cab",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 2,
    "execution_start": 1667410294413,
    "source_hash": "3f61b438",
    "tags": []
   },
   "outputs": [],
   "source": [
    "centroids2 = kmeans.cluster_centers_"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 32,
   "metadata": {
    "cell_id": "9fd32deb019246c9b4ed5b94f3e65432",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 6,
    "execution_start": 1667410300116,
    "source_hash": "91c7678d",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/plain": [
       "array([[-6.25246917, -9.60729483],\n",
       "       [-4.51866555, -1.17493755],\n",
       "       [-9.53103386, -5.24094253]])"
      ]
     },
     "execution_count": 32,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "centroids2"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Let's make a DataFrame with that data."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 33,
   "metadata": {
    "cell_id": "d3d13872d4414fd0b0c86e61eae2353b",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 2,
    "execution_start": 1667410331588,
    "source_hash": "37ea87de",
    "tags": []
   },
   "outputs": [],
   "source": [
    "df_centroids2 = pd.DataFrame(centroids2, columns=[\"x\",\"y\"])"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Let's see the new positions of these centroids.  If you scroll up and look at our clusters that we found by hand on Monday, and look at the centers of those clusters, they are exactly the points that are drawn below."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 34,
   "metadata": {
    "cell_id": "b9e4de2fb5d54bada5c923cda5a1d6b0",
    "deepnote_cell_type": "code",
    "deepnote_to_be_reexecuted": false,
    "execution_millis": 222,
    "execution_start": 1667410352901,
    "source_hash": "8ba164c1",
    "tags": []
   },
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<div id=\"altair-viz-73fd2b8bc545472e96c2375c2074c587\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-73fd2b8bc545472e96c2375c2074c587\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-73fd2b8bc545472e96c2375c2074c587\");\n",
       "    }\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm//vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm//vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm//vega-lite@4.17.0?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm//vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      require([\"vega-embed\"], displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"4.17.0\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 400, \"continuousHeight\": 300}}, \"layer\": [{\"data\": {\"name\": \"data-12bb0cfa88e3591c947140beaa35a2c9\"}, \"mark\": \"circle\", \"encoding\": {\"color\": {\"field\": \"cluster-sklearn\", \"type\": \"nominal\"}, \"x\": {\"field\": \"x\", \"type\": \"quantitative\"}, \"y\": {\"field\": \"y\", \"type\": \"quantitative\"}}}, {\"data\": {\"name\": \"data-2fced9028f86884751847543225100fe\"}, \"mark\": {\"type\": \"point\", \"filled\": true, \"shape\": \"cross\", \"size\": 500, \"stroke\": \"black\"}, \"encoding\": {\"x\": {\"field\": \"x\", \"type\": \"quantitative\"}, \"y\": {\"field\": \"y\", \"type\": \"quantitative\"}}}], \"$schema\": \"https://vega.github.io/schema/vega-lite/v4.17.0.json\", \"datasets\": {\"data-12bb0cfa88e3591c947140beaa35a2c9\": [{\"x\": -6.923241645991934, \"y\": -10.669531965589794, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -8.630620334313644, \"y\": -7.13940563765234, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.630480689769236, \"y\": -2.7204493549869673, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.3064765942034278, \"y\": 5.307976760491872, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.570053657727669, \"y\": -3.0144649054564128, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.0005101098483093, \"y\": 2.779051528226904, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -4.818268385973806, \"y\": -2.772148223435087, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.339647992855507, \"y\": -1.276257644021504, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.943088401134986, \"y\": -3.899939010564857, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -5.549245254367849, \"y\": -3.412989678624256, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.145089897229651, \"y\": -9.544921980678078, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -7.09669935635653, \"y\": -8.04074036220985, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.826415121664895, \"y\": -1.963461960474791, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.8319881134989553, \"y\": 3.5286314509217895, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.342672348933138, \"y\": -3.1654648218447767, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -7.340728246218882, \"y\": -6.924272518973348, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -7.946539060795657, \"y\": -3.3676865471908224, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -8.245985356429562, \"y\": -8.613158209058627, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -1.9819771099620271, \"y\": 4.022435514174746, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -4.350980346475657, \"y\": -3.6947667796776424, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.476869602439182, \"y\": -3.6031813885349258, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -11.01959844829441, \"y\": -3.1588203077621224, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.17255903986124, \"y\": -4.318359712685473, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.406718199699357, \"y\": 6.098944469870908, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.721494975923703, \"y\": -2.8844080642178818, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.58935963436387, \"y\": -4.43379547940742, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.4612601858217498, \"y\": 4.525498514650621, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.9190034548522448, \"y\": 3.452789267035227, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.409351666841584, \"y\": -2.674820464521483, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.367228090988169, \"y\": -3.326660724595877, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.72766124534928, \"y\": -7.145162671548282, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -2.279560749896649, \"y\": 5.104521902915381, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.848875595943767, \"y\": -3.0397050619974144, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.079930511938528, \"y\": -7.081975676778154, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.266829294846004, \"y\": -2.696450548014737, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.05367511988283, \"y\": -9.629790765144225, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -10.08222053105773, \"y\": -4.250710430136708, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -11.870873481288402, \"y\": -3.0327334293082484, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.371073070369658, \"y\": -7.9563583253737535, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -9.375908999347528, \"y\": -4.553153078434052, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.6340198681573215, \"y\": -2.583403556386127, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.54609655309786, \"y\": -2.8491742210599913, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.6982554152145706, \"y\": 2.7907175141398666, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.602176024325104, \"y\": -0.6599084896160656, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.03429022029922, \"y\": -4.088211960497224, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.372307836118383, \"y\": -8.631900461039706, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.226478332523264, \"y\": -2.339987172800089, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.956781476027413, \"y\": -3.979057011247294, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.4270653467697816, \"y\": 5.089041275707647, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.6207353040891055, \"y\": 6.59346951581742, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.32810279336092774, \"y\": 4.119182005367143, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.62305450511909, \"y\": -4.547191614617715, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.12674270112086, \"y\": -4.461805681638605, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.241344969182345, \"y\": -3.2350587337503987, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.199675312953462, \"y\": -7.104009813871477, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.113697689125498, \"y\": -4.128807516024957, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.341613212863495, \"y\": -4.95351773723572, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -12.504153156329163, \"y\": -6.067512469280031, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.323316398832233, \"y\": -4.675740452777355, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.320330019823476, \"y\": -2.733500952132815, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.205334073931087, \"y\": 4.207652005954265, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.5279305184970926, \"y\": 5.92630668526536, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.874300341631166, \"y\": -3.648081507119194, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.66948545344214, \"y\": -4.260598838360167, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.3739725806942609, \"y\": 5.291631033113889, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.600857078206719, \"y\": -3.1196968767431805, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.991754122866363, \"y\": -8.335648510987143, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.224478686299447, \"y\": -2.438462244886699, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -11.105425005937654, \"y\": -3.9710668652733467, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.957623350499006, \"y\": -4.87178858566323, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.654616438260894, \"y\": -7.293357132827133, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.95313776602679, \"y\": -3.367438116260589, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.113510079104775, \"y\": -2.0188199201889114, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.24251437964129, \"y\": -9.663684479711614, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.349294557744197, \"y\": -3.545773316732582, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.2926133215005216, \"y\": -3.6889242583746578, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.353879526667555, \"y\": -8.545044344432092, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.784234728636943, \"y\": -4.484068484204371, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.662563249318529, \"y\": -2.343900922698393, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.608933107282834, \"y\": -4.614692790516223, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -0.25201990570729094, \"y\": 4.535591448553887, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.9274479855745354, \"y\": 4.9368453355813475, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.413065893175675, \"y\": -3.6290743039147837, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -0.8486082331449638, \"y\": 5.4509319635504205, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.8667530400291719, \"y\": 3.7829591391633697, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.184291497164567, \"y\": -4.010173028249052, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.931929183434267, \"y\": -5.424505473195829, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -2.7544717516895614, \"y\": 4.575872301901829, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -11.717106978272625, \"y\": -3.8962275512811124, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.850812127253619, \"y\": -4.00305112652188, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.3439249587069142, \"y\": 2.3842886530191585, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.1620365439131, \"y\": -7.314593357395667, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.188868143387419, \"y\": -2.163593864156933, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -7.132292597186354, \"y\": -4.022967302286994, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -0.04261030709121938, \"y\": 4.909230750843066, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.244494478309097, \"y\": -7.65150299527888, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -8.137846462369383, \"y\": -7.658069493423728, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.7345134534846505, \"y\": -1.3833019413836023, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.963694242977034, \"y\": -9.270338797063157, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": 0.8687658008578785, \"y\": 4.1578550911277015, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.5451769291687276, \"y\": 3.819965930021569, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.016944284363998, \"y\": -8.6713736570228, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -3.3337557135674514, \"y\": 5.231519693282148, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -11.43858852794494, \"y\": -2.7210954765347686, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.5208762667245614, \"y\": 5.081201394531134, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.8439444259585525, \"y\": -4.150582220449893, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -5.876197384285145, \"y\": -3.2807891557700892, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.2181954623799376, \"y\": 4.3063346355596615, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.0034135815195953, \"y\": 4.450086725677409, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.107703993816708, \"y\": -3.94479960192718, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.030458537548547, \"y\": -1.237347564565535, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.9568513663453935, \"y\": -8.12381048565328, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -2.3302221922375503, \"y\": 4.784053655164081, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.984359827305925, \"y\": -4.648042142650431, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.3308060367853387, \"y\": 4.39382526992426, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.779624186339902, \"y\": -4.390857530191786, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.034844856038634, \"y\": 3.7677594613889394, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.167442445959466, \"y\": -3.249983778680282, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -0.49922133595474993, \"y\": 4.775982585218885, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.766811435831203, \"y\": -3.4128177881590513, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.699056929910277, \"y\": -4.490571571238543, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.2872962121876474, \"y\": -6.683067760798305, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -8.178318286630729, \"y\": -8.220638129570899, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -9.144431281697479, \"y\": -4.366377857074236, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.223235428900379, \"y\": -3.5122637563645807, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -9.712964392804043, \"y\": -3.690881101899936, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -0.03190915279961315, \"y\": 4.744501565417638, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.104060439867236, \"y\": -8.381982275844257, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.5248250144984254, \"y\": -7.508874438593723, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.31161342896361, \"y\": -2.9764169703468255, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.5381421977128555, \"y\": 4.815390406779989, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.580410498678239, \"y\": -3.168577896347615, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.53106923642426, \"y\": -2.9196616794237036, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.765022329984836, \"y\": -3.278777840407201, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.546589564918502, \"y\": -4.648269448548444, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.3939337329988195, \"y\": -6.806122639953547, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -2.9915115724657677, \"y\": 2.645801308444378, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.675582536987324, \"y\": -4.559022548160358, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -3.5175417703952716, \"y\": 5.642653899295982, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.9985396183498266, \"y\": 6.198648077818729, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.964979013839094, \"y\": -2.0374646912902934, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.852795074857454, \"y\": -7.791380788784477, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -4.643104258815785, \"y\": -2.227894218120615, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.359389585992692, \"y\": 4.054240022349643, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.5257904636130821, \"y\": 3.3065986015291307, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.1563750925131935, \"y\": 5.6997157513754235, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.425300098097034, \"y\": -2.173286191076472, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.701833046489826, \"y\": -2.630838382829303, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.046329708889857, \"y\": -6.92266990466932, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -8.145592876103246, \"y\": -7.42775410014251, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.156850945511998, \"y\": -4.056235764936742, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.161707784218832, \"y\": -2.409989436558721, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.4686444212810534, \"y\": 6.506745005322004, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.746727983979049, \"y\": -8.172459738721683, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.9860594040267037, \"y\": 3.0638140794657254, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.32899567358573, \"y\": -3.5668094022301133, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.343132351066401, \"y\": -4.004536994970869, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.559546156361863, \"y\": -2.831020225706112, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.165911343397818, \"y\": -4.127528891671195, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.841448653383019, \"y\": -4.143569572448066, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.276810225196392, \"y\": -2.330499461409204, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.103057216160643, \"y\": -3.323152876678294, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.902287416461844, \"y\": -3.031898478703693, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.721213198830919, \"y\": -4.686620151258517, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.8513954583101344, \"y\": 3.5188609047583252, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.6932118924458734, \"y\": -6.300218619554666, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.533718390201513, \"y\": -8.149227262648669, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -8.463695002324215, \"y\": -8.071460285818429, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.750045280149759, \"y\": -3.565909665608377, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.1710417594110225, \"y\": 4.330918155822106, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.526285788379845, \"y\": -8.669576013270067, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -9.23890684355517, \"y\": -3.0684397328983213, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.128030509059693, \"y\": -2.516980580274055, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.10406450569264, \"y\": -7.420204872904966, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.6158909127701304, \"y\": 4.180175626329556, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.987585326318378, \"y\": -3.0333306141154415, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -11.941035909321455, \"y\": -3.6008541829938716, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.439941783166274, \"y\": -3.6298211937617726, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -11.424267924173634, \"y\": -2.185388599052178, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.009929142269778, \"y\": -9.068652472855751, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.474356490442799, \"y\": -3.7433886276640225, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.631380489113043, \"y\": -4.9979379292709964, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": 0.5260155005846419, \"y\": 3.009993533355024, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.763243925323785, \"y\": -9.366566230452749, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.2796552640230505, \"y\": -8.818095871591257, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -9.468832758067927, \"y\": -6.190435058475675, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.773366182553252, \"y\": -3.5673995330002293, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.692425332146296, \"y\": -8.301717907683438, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.444399699191618, \"y\": -9.168031795482715, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -7.11478468705883, \"y\": -5.3869913396768965, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -0.3858039757897429, \"y\": 6.373591618097896, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.004547116103499, \"y\": 4.175650125707692, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.755176277975085, \"y\": -9.30821073955341, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -9.141684205581292, \"y\": -7.205726937278369, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.920925345563742, \"y\": -3.2757404756717943, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.351220657673829, \"y\": 4.0097363419871845, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.919078510870565, \"y\": -2.2391986140442226, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.6220052551614295, \"y\": -8.692909666826537, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.54246303831446, \"y\": -8.127228110438386, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -2.4139578469451726, \"y\": 5.659358024076449, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.371515957997354, \"y\": -8.911295433180277, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -12.14017923273047, \"y\": -4.783517411385913, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -4.45264490609506, \"y\": 6.344018682459241, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.596988201326771, \"y\": -4.19535852653093, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.075036217253955, \"y\": -2.156064045450953, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.248282380977763, \"y\": -7.052227901955563, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -4.778911006562106, \"y\": -2.4133316499348325, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -12.411215504826888, \"y\": -5.730914923169192, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.7526434946447385, \"y\": -8.346549745098049, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.054921393415961, \"y\": -4.222577494532622, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.382544752485892, \"y\": -2.4952403087536847, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.225705023497352, \"y\": -3.7931357906625034, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -11.949817790898333, \"y\": -5.355677692816341, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.628670918240042, \"y\": -8.063541697301455, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -4.617671134403657, \"y\": -1.6711114461396843, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.122196637435978, \"y\": -3.3130212312625433, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.29225072428276, \"y\": -2.357382942013979, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": 0.00024227116135100424, \"y\": 5.148534029420497, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.79988166193185, \"y\": -2.2487543759907975, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -2.7768702545837973, \"y\": 4.640905566660254, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.396949787558266, \"y\": -3.769637033156362, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.9226308111314205, \"y\": -7.639722624635498, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -11.576868789047566, \"y\": -4.78197652621961, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.668247372033288, \"y\": -3.8260750868989764, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -11.157882608635378, \"y\": -2.6032417286814784, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.473085378791083, \"y\": -3.4757383655456473, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.981184936347486, \"y\": -3.776160827087244, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.0410207751622265, \"y\": 3.9633179381425236, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.328560152872887, \"y\": -2.6089330879852275, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.1389835688751488, \"y\": 3.262148479220019, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.179056384691831, \"y\": -7.9633664552409495, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.235654439405284, \"y\": -2.798060661274979, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.771332562994431, \"y\": -8.592225768290323, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -9.145008441927814, \"y\": -3.9179884503084472, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.6173461592329268, \"y\": 4.9893050825589835, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.7786753045082477, \"y\": 6.362568772546803, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.54642848761293, \"y\": -5.637408533231442, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.914865904929935, \"y\": -7.6896937828923795, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.8461296778558682, \"y\": 4.304743996314016, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.5283458556366405, \"y\": -8.1536031133824, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -6.009153369932298, \"y\": -3.3492515180044244, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.546283243062167, \"y\": -4.571385395044784, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.316556391289163, \"y\": -7.770512930899681, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.20423398910866, \"y\": -8.881765594048746, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -7.556007321448557, \"y\": -8.018854988584291, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.67856791941142, \"y\": -7.605098519935852, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.214468263289775, \"y\": -4.799953118763537, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.376629802814094, \"y\": -2.9972268432385083, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.318447094429766, \"y\": -8.92829839169748, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.827884398968216, \"y\": -4.833926147207375, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.065699095871062, \"y\": -1.5337694648059559, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.346732606068119, \"y\": 3.561284227344442, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.2560682641577197, \"y\": 5.0000683920802285, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.839797448183742, \"y\": -2.178361856135961, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.8708821070342445, \"y\": -2.227162357797841, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.7960046498445525, \"y\": 4.287435681068351, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.379726969732767, \"y\": -4.13752487301695, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.236059371477093, \"y\": -4.547109919532199, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -10.279448772718714, \"y\": -1.8969930241649111, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.4168904648667249, \"y\": 4.608320047717165, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.780454121464204, \"y\": -4.582979224413729, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": 0.08525185826796045, \"y\": 3.6452829679480585, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.202686409228257, \"y\": -4.327786866206425, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.568186363631137, \"y\": -4.560346945563251, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.1643485819705592, \"y\": 4.2317867091006605, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.163458506063613, \"y\": -3.108308017459318, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.321525640143877, \"y\": -9.662800788848088, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -7.520999738806424, \"y\": -9.133118356285845, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -9.220293298404238, \"y\": -4.0721197194662935, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.849168160183929, \"y\": -2.9524671236829034, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.863664307098478, \"y\": -2.751293685163499, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.797152237582278, \"y\": -3.458041360968013, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.979490066070415, \"y\": 4.08668826994449, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.060438097737446, \"y\": 5.230495487293569, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.668391828142745, \"y\": -0.7950678467818668, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.757969185355724, \"y\": 4.908984207745029, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.420569465043878, \"y\": -3.866884140838917, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.124250086206238, \"y\": -6.704238700465809, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.3788948288258804, \"y\": 4.333377165095884, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.614664441619893, \"y\": -7.525791021870934, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.340520809891421, \"y\": 4.157119493365752, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.211599997834005, \"y\": -8.292939835065118, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.5688561328000565, \"y\": -8.135272212973218, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -1.7700069323658358, \"y\": 3.789127805130801, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.365858337613077, \"y\": -7.345772187618775, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.499522836500689, \"y\": 5.282658789959386, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.8588279382837056, \"y\": 5.2698351863871835, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.738849346027174, \"y\": -3.2432766529495733, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -10.820179692557353, \"y\": -3.231637255250512, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.536820123190024, \"y\": -3.3608757459391847, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -12.034913729333896, \"y\": -5.895937728839661, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.269109092135366, \"y\": -2.7352182431364156, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.712996039785251, \"y\": -2.9032498381724894, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.361186337903108, \"y\": -2.726983818567998, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -5.489414275085358, \"y\": -6.946620206588477, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": 0.5311398230212139, \"y\": 2.510128945952069, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.6412677543087515, \"y\": -7.249228930620518, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -9.482638890013735, \"y\": -6.735883015016127, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.531037040483667, \"y\": -6.768236760975406, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.310785954173774, \"y\": -2.051746477365137, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.70233177666608, \"y\": -4.194625400586082, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.110130706597376, \"y\": -2.3106112800394785, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.839726334384508, \"y\": -9.206774178324359, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -11.753638143885953, \"y\": -3.2385589498773895, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.291994815507753, \"y\": -9.852561709346341, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -7.855682137432907, \"y\": -6.9295058926639355, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.196710687456825, \"y\": -2.0868771722110795, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.963565376976385, \"y\": -7.833571162679451, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.776804016652588, \"y\": -6.655119921047011, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.87499402257129, \"y\": -4.82113577023613, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.84048020802667, \"y\": 3.802569241049904, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.980674032726418, \"y\": -8.56048014764401, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.32066245851635, \"y\": -3.3075189181067537, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.179799662377421, \"y\": -3.0080344705160247, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.1766543638687477, \"y\": 3.4094630411928986, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.6732247181272776, \"y\": 4.620023770178603, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.938921711708721, \"y\": -3.5152140767725673, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.489374974228666, \"y\": -8.884759086354379, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -2.8964132824899167, \"y\": 5.282328796492461, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.8133992583634861, \"y\": 3.546973928011175, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.7775266738006055, \"y\": -2.851452759799448, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.248838499333678, \"y\": -8.765635077752869, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -3.103673711378109, \"y\": 3.90202400589671, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.0572406256528906, \"y\": 4.826772072490221, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.732150477880266, \"y\": -5.046954544738055, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -9.936962307853594, \"y\": -3.742223792355488, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -3.0326772261952755, \"y\": 4.721649260885453, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.703553011055385, \"y\": -2.760662479608511, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.684756308285753, \"y\": -3.768169242136279, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.62182374152277, \"y\": -8.76567023290839, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.671772938803755, \"y\": -9.977147957249239, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -1.9257784119521415, \"y\": 4.43910441553652, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.162994877003415, \"y\": -3.388965691868891, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -3.743803427487374, \"y\": -8.75345343500136, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.666012113538473, \"y\": -4.970196326305104, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -2.8896180416154813, \"y\": 4.957027364819377, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.3599584108697744, \"y\": 4.203095419878603, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.8049155726466335, \"y\": -3.4960254831509756, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -7.104806758014256, \"y\": -4.108305308544761, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.9668553912810705, \"y\": -3.128763922329747, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.313544948497254, \"y\": -8.012832668176694, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -4.471206790960085, \"y\": -3.5413104324230704, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.5394009534668904, \"y\": 5.023692978550581, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.6087521511724905, \"y\": 3.769494222273808, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.192769844529003, \"y\": -3.147955123131023, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.8020781039706595, \"y\": 4.057147146430284, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": 0.24509880239246162, \"y\": 5.517546567403667, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -3.31028117177262, \"y\": 3.515934284680113, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.841878032458153, \"y\": 3.740735350644035, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.75867611852229, \"y\": -8.75783107378842, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.995910561456476, \"y\": -8.112856673601206, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -4.9836068695957065, \"y\": -3.2052296074107085, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.8684541393232976, \"y\": 4.993113060025359, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.715036791404682, \"y\": -4.779445979210525, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.473733222730479, \"y\": -2.7868254137329, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.992630284501854, \"y\": -7.143440769676486, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.5377386349570825, \"y\": 5.535973776755081, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.446450536324724, \"y\": -4.625796586149801, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.0967988093294934, \"y\": 4.647226961080894, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.252568773979549, \"y\": -2.9168283269714537, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.19745196890354544, \"y\": 2.3463491593455075, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.067041200343585, \"y\": -4.0617406145367445, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.13468589114183, \"y\": -4.507934236283532, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -10.372517190955755, \"y\": -4.703318164638871, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.8818880485588787, \"y\": 4.205731799913164, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.154984836988162, \"y\": -3.1077859806863612, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.142547993526798, \"y\": -3.652022063256501, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.427494271234997, \"y\": -9.6383845559495, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -1.1300945815958103, \"y\": 4.544191078840548, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.284855048319109, \"y\": -8.782669708677824, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.3332534877783155, \"y\": -8.284903730911063, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.403201107191756, \"y\": -7.166875924411981, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.2218758561510565, \"y\": -9.488430827639236, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.098342933428682, \"y\": -7.440179054320776, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -7.208077928664976, \"y\": -7.120244325674294, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -9.687440215869334, \"y\": -6.047596363586017, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.873729380129867, \"y\": -7.595788651766012, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -1.1466300855305107, \"y\": 4.108397033740446, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.903442201983345, \"y\": -8.180757490892024, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -2.760179083161441, \"y\": 5.551213578682775, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.2360655518844066, \"y\": 4.483829937191721, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.97584966702655, \"y\": -4.422022363641096, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -2.1066884707345177, \"y\": 5.63099757364866, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -4.73558876279152, \"y\": -4.237489688775991, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.723309622787529, \"y\": -4.821117224986795, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.26074369398222, \"y\": -5.647247819305372, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.883843439299868, \"y\": -7.046052653674982, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -2.1577734695795976, \"y\": 4.095504890540311, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.859884436213619, \"y\": -4.73888254428344, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -0.46064202604701254, \"y\": 4.591646286327105, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.0568548716452355, \"y\": -5.029466423503938, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.660550064568029, \"y\": -8.462349423265907, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -8.419239819175033, \"y\": -3.458347884476478, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -10.994732299182784, \"y\": -4.06014253414243, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.7137652871467735, \"y\": -8.221998567702684, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.797259994344861, \"y\": -4.244943142864037, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.237463277204007, \"y\": -4.014001039138014, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -2.932118661194514, \"y\": 4.720037589306593, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.6614513887865232, \"y\": 3.0098694442477294, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.657343468811008, \"y\": -10.458136042394184, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -9.980547777279012, \"y\": -4.382490825598298, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.519403743255727, \"y\": -2.3878033376158005, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.9696766794665796, \"y\": 1.9716521047360227, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -3.884649810113071, \"y\": -2.843362609949925, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.82969906210201, \"y\": -2.9906732129170535, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.667001758775887, \"y\": -9.149238993712444, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -6.628895987361329, \"y\": -8.840715497684219, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -6.489449614672605, \"y\": -2.0675373330401436, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.171342305213279, \"y\": -10.944224511317891, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -11.30424657649712, \"y\": -3.8769680672036126, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.536548403676084, \"y\": -5.12933121880452, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.098661321079398, \"y\": -7.427311252103224, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -8.789256175101139, \"y\": -2.837646744578718, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -7.323865037058271, \"y\": -7.963934907065387, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.033080428531813, \"y\": -1.842743493762495, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.0361977293704867, \"y\": 3.971533185887934, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.428298774243108, \"y\": -6.743974715260037, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -2.8793043029275522, \"y\": 6.855858517904559, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.52994652455128, \"y\": -2.835215150645509, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.114230780704071, \"y\": -3.2089354347677936, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.7824501314671677, \"y\": 3.4707204345840927, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.952718093979204, \"y\": -3.344833848878355, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.166179014456907, \"y\": -3.7917058620492132, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.6421504976485148, \"y\": 3.284471140096273, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.335342960043219, \"y\": -7.870232570813044, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.311077061000285, \"y\": -3.921180806827734, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.7800244777370269, \"y\": 3.1733691333967347, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.6841768618242978, \"y\": 3.631328249674002, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.555207225439144, \"y\": -3.0141798031667166, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.343540093678463, \"y\": -2.138976643927715, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -11.536505749397744, \"y\": -4.401243732079323, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -4.895037580840365, \"y\": -2.486334557899435, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.443969898035991, \"y\": -8.959412922006269, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -1.5817387780358136, \"y\": 5.024870130841686, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.0299385942939985, \"y\": -6.699310523015227, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.170742382737493, \"y\": -2.5607820370672085, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -2.2218653369676096, \"y\": 6.361367943852196, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.5738544614806, \"y\": -8.319714059510003, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -7.658225937900157, \"y\": -7.642920507878789, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.895012928310167, \"y\": -9.317236075785248, \"cluster\": 0, \"cluster-sklearn\": 0}, {\"x\": -11.114182521537845, \"y\": -3.872421446389926, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -0.794152276623841, \"y\": 2.104951171962879, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.428031934213674, \"y\": -5.521293971786157, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -5.897807017657774, \"y\": -8.192896804714527, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -6.5916969689749285, \"y\": -2.4477995927555383, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.457857759408556, \"y\": -3.3098143609273802, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.775571330280513, \"y\": -2.837507439439036, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -10.234149455466088, \"y\": -3.2255350473635556, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.266818388358796, \"y\": -8.255160143887645, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.205809800827664, \"y\": -3.2985383945615987, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.460452643297657, \"y\": -2.3083155293806703, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.0425995184711425, \"y\": -3.453323508845682, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.099628043228513, \"y\": -3.142269151874201, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.660069496619134, \"y\": -3.437769650054597, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.080973984400039, \"y\": -3.039723766765942, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.412647124911402, \"y\": -6.682488247932676, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.365134104901397, \"y\": -1.3885973124935187, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.416650401331225, \"y\": -4.4325334624906105, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.416238539567072, \"y\": -8.045884810800455, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -5.889193480102257, \"y\": -2.370494723384702, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.4294651705841361, \"y\": 5.168501049155187, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.561180687841756, \"y\": -3.959673108791006, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -1.4729985149602352, \"y\": 4.816541516051418, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.881008041197264, \"y\": -3.316926149839501, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -10.412559404941337, \"y\": -3.5014025100419817, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.552093769106948, \"y\": -3.158409998594174, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -0.7906737489426516, \"y\": 5.156901508708169, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.10075436532866022, \"y\": 4.515892565268589, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.3090139272844121, \"y\": 3.09420645760573, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.547556990885923, \"y\": -2.188013446844133, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -5.320300105898895, \"y\": -2.99303868802891, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.48229870186986, \"y\": -5.068219599729142, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.743616269173907, \"y\": -8.87844303036941, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.25189237126047, \"y\": -2.5535046048822188, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -1.9657639185768914, \"y\": 5.234464511450407, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -5.88036774243447, \"y\": -2.363262900885486, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.347745735292583, \"y\": -8.419554988942766, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -0.7587039566841077, \"y\": 3.7227620096688283, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -8.413578630454982, \"y\": -6.850692568137864, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -0.8205764920740146, \"y\": 5.337591950146718, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -7.934890406848591, \"y\": -7.784037635077171, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.694465659843353, \"y\": -4.062053044388285, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -0.8576988740771748, \"y\": 4.453057167267307, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": 0.15097500817693676, \"y\": 3.100762946285424, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -6.553944409165649, \"y\": -6.44256626595291, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -10.931627159279603, \"y\": -4.4863688734468035, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -6.501555964616166, \"y\": -4.653293312950241, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.9365051935017945, \"y\": -6.392812921048723, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -10.13368982975369, \"y\": -4.750618332856334, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -9.891489779617883, \"y\": -5.479028863592562, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.898716170905923, \"y\": -4.854983037839203, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -8.113949928826354, \"y\": -7.836569209467391, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -5.290783535506342, \"y\": -3.6484668810576015, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -1.410760744560749, \"y\": 4.109848716472299, \"cluster\": 1, \"cluster-sklearn\": 1}, {\"x\": -9.505375947716452, \"y\": -4.63402668837209, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.827494555623418, \"y\": -2.5103210388828026, \"cluster\": 1, \"cluster-sklearn\": 2}, {\"x\": -6.380880855475786, \"y\": -8.506638087491059, \"cluster\": 1, \"cluster-sklearn\": 0}, {\"x\": -8.960149131134205, \"y\": -8.063498987179393, \"cluster\": 2, \"cluster-sklearn\": 2}, {\"x\": -7.666038984307122, \"y\": -7.59715459454666, \"cluster\": 2, \"cluster-sklearn\": 0}, {\"x\": -6.465344070108034, \"y\": -2.855446332878923, \"cluster\": 1, \"cluster-sklearn\": 1}], \"data-2fced9028f86884751847543225100fe\": [{\"x\": -6.2524691712775216, \"y\": -9.607294827209431}, {\"x\": -4.518665554564543, \"y\": -1.1749375486641764}, {\"x\": -9.531033856967191, \"y\": -5.240942527532808}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.LayerChart(...)"
      ]
     },
     "execution_count": 34,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "c1 = alt.Chart(df_data).mark_circle().encode(\n",
    "    x=\"x\",\n",
    "    y=\"y\",\n",
    "    color=\"cluster-sklearn:N\"\n",
    ")\n",
    "\n",
    "c2 = alt.Chart(df_centroids2).mark_point(\n",
    "    size=500,\n",
    "    shape='cross',\n",
    "    filled=True,\n",
    "    stroke=\"black\",\n",
    ").encode(\n",
    "    x=\"x\",\n",
    "    y=\"y\",\n",
    ")\n",
    "\n",
    "c1+c2"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {
    "cell_id": "a59aef5821f64f4e833ce283fbc068e9",
    "deepnote_cell_type": "code",
    "tags": []
   },
   "source": [
    "The next step in the clustering algorithm would be to move these centers to the centers of the current clusters (for example, the cross in the red cluster would definitely move up, because there are so many more red points above it than below it).  The next step after that would be to assign new clusters, based on what centroid is closest.  (I use the words *centroid*, *center*, *mean*, and *average* interchangeably.)\n",
    "\n",
    "Worksheet 10 is a little longer than our usual worksheets, but it has a very nice payoff, where at the end of Worksheet 10, you will have a chart with a slider you can drag to illustrate the K-means clustering algorithm."
   ]
  }
 ],
 "metadata": {
  "deepnote": {},
  "deepnote_execution_queue": [],
  "deepnote_notebook_id": "e3205ee0c38040a3ae506cf8d38ce529",
  "deepnote_persisted_session": {
   "createdAt": "2022-11-02T03:01:43.602Z"
  },
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.9.13"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
